@@ -18,52 +18,62 @@
 //! Execution plan for reading line-delimited Avro files
 
 use std::any::Any;
+use std::fmt::Formatter;
 use std::sync::Arc;
 
-use super::FileScanConfig;
+use super::FileOpener;
+#[cfg(feature = "avro")]
+use crate::datasource::avro_to_arrow::Reader as AvroReader;
+
 use crate::error::Result;
-use crate::physical_plan::metrics::{ExecutionPlanMetricsSet, MetricsSet};
-use crate::physical_plan::{
-    DisplayAs, DisplayFormatType, ExecutionPlan, Partitioning, PlanProperties,
-    SendableRecordBatchStream, Statistics,
-};
 
 use arrow::datatypes::SchemaRef;
-use datafusion_execution::TaskContext;
-use datafusion_physical_expr::{EquivalenceProperties, LexOrdering};
+use datafusion_common::{Constraints, Statistics};
+use datafusion_datasource::file::FileSource;
+use datafusion_datasource::file_scan_config::FileScanConfig;
+use datafusion_datasource::source::DataSourceExec;
+use datafusion_execution::{SendableRecordBatchStream, TaskContext};
+use datafusion_physical_expr::{EquivalenceProperties, Partitioning};
+use datafusion_physical_expr_common::sort_expr::LexOrdering;
 use datafusion_physical_plan::execution_plan::{Boundedness, EmissionType};
+use datafusion_physical_plan::metrics::{ExecutionPlanMetricsSet, MetricsSet};
+use datafusion_physical_plan::{
+    DisplayAs, DisplayFormatType, ExecutionPlan, PlanProperties,
+};
+
+use object_store::ObjectStore;
 
 /// Execution plan for scanning Avro data source
 #[derive(Debug, Clone)]
+#[deprecated(since = "46.0.0", note = "use DataSourceExec instead")]
 pub struct AvroExec {
+    inner: DataSourceExec,
     base_config: FileScanConfig,
-    projected_statistics: Statistics,
-    projected_schema: SchemaRef,
-    projected_output_ordering: Vec<LexOrdering>,
-    /// Execution metrics
-    metrics: ExecutionPlanMetricsSet,
-    cache: PlanProperties,
 }
 
+#[allow(unused, deprecated)]
 impl AvroExec {
     /// Create a new Avro reader execution plan provided base configurations
     pub fn new(base_config: FileScanConfig) -> Self {
-        let (projected_schema, projected_statistics, projected_output_ordering) =
-            base_config.project();
+        let (
+            projected_schema,
+            projected_constraints,
+            projected_statistics,
+            projected_output_ordering,
+        ) = base_config.project();
         let cache = Self::compute_properties(
             Arc::clone(&projected_schema),
             &projected_output_ordering,
+            projected_constraints,
             &base_config,
         );
+        let base_config = base_config.with_source(Arc::new(AvroSource::default()));
         Self {
+            inner: DataSourceExec::new(Arc::new(base_config.clone())),
             base_config,
-            projected_schema,
-            projected_statistics,
-            projected_output_ordering,
-            metrics: ExecutionPlanMetricsSet::new(),
-            cache,
         }
     }
+
     /// Ref to the base configs
     pub fn base_config(&self) -> &FileScanConfig {
         &self.base_config
@@ -73,10 +83,12 @@ impl AvroExec {
     fn compute_properties(
         schema: SchemaRef,
         orderings: &[LexOrdering],
+        constraints: Constraints,
         file_scan_config: &FileScanConfig,
     ) -> PlanProperties {
         // Equivalence Properties
-        let eq_properties = EquivalenceProperties::new_with_orderings(schema, orderings);
+        let eq_properties = EquivalenceProperties::new_with_orderings(schema, orderings)
+            .with_constraints(constraints);
         let n_partitions = file_scan_config.file_groups.len();
 
         PlanProperties::new(
@@ -88,17 +100,14 @@ impl AvroExec {
     }
 }
 
+#[allow(unused, deprecated)]
 impl DisplayAs for AvroExec {
-    fn fmt_as(
-        &self,
-        t: DisplayFormatType,
-        f: &mut std::fmt::Formatter,
-    ) -> std::fmt::Result {
-        write!(f, "AvroExec: ")?;
-        self.base_config.fmt_as(t, f)
+    fn fmt_as(&self, t: DisplayFormatType, f: &mut Formatter) -> std::fmt::Result {
+        self.inner.fmt_as(t, f)
     }
 }
 
+#[allow(unused, deprecated)]
 impl ExecutionPlan for AvroExec {
     fn name(&self) -> &'static str {
         "AvroExec"
@@ -109,20 +118,17 @@ impl ExecutionPlan for AvroExec {
     }
 
     fn properties(&self) -> &PlanProperties {
-        &self.cache
+        self.inner.properties()
     }
-
     fn children(&self) -> Vec<&Arc<dyn ExecutionPlan>> {
         Vec::new()
     }
-
     fn with_new_children(
         self: Arc<Self>,
         _: Vec<Arc<dyn ExecutionPlan>>,
     ) -> Result<Arc<dyn ExecutionPlan>> {
         Ok(self)
     }
-
     #[cfg(not(feature = "avro"))]
     fn execute(
         &self,
@@ -133,76 +139,154 @@ impl ExecutionPlan for AvroExec {
             "Cannot execute avro plan without avro feature enabled".to_string(),
         ))
     }
-
     #[cfg(feature = "avro")]
     fn execute(
         &self,
         partition: usize,
         context: Arc<TaskContext>,
     ) -> Result<SendableRecordBatchStream> {
-        use super::file_stream::FileStream;
-        let object_store = context
-            .runtime_env()
-            .object_store(&self.base_config.object_store_url)?;
-
-        let config = Arc::new(private::AvroConfig {
-            schema: Arc::clone(&self.base_config.file_schema),
-            batch_size: context.session_config().batch_size(),
-            projection: self.base_config.projected_file_column_names(),
-            object_store,
-        });
-        let opener = private::AvroOpener { config };
-
-        let stream =
-            FileStream::new(&self.base_config, partition, opener, &self.metrics)?;
-        Ok(Box::pin(stream))
+        self.inner.execute(partition, context)
     }
 
     fn statistics(&self) -> Result<Statistics> {
-        Ok(self.projected_statistics.clone())
+        self.inner.statistics()
     }
 
     fn metrics(&self) -> Option<MetricsSet> {
-        Some(self.metrics.clone_inner())
+        self.inner.metrics()
     }
 
     fn fetch(&self) -> Option<usize> {
-        self.base_config.limit
+        self.inner.fetch()
     }
 
     fn with_fetch(&self, limit: Option<usize>) -> Option<Arc<dyn ExecutionPlan>> {
-        let new_config = self.base_config.clone().with_limit(limit);
+        self.inner.with_fetch(limit)
+    }
+}
 
-        Some(Arc::new(Self {
-            base_config: new_config,
-            projected_statistics: self.projected_statistics.clone(),
-            projected_schema: Arc::clone(&self.projected_schema),
-            projected_output_ordering: self.projected_output_ordering.clone(),
-            metrics: self.metrics.clone(),
-            cache: self.cache.clone(),
-        }))
+/// AvroSource holds the extra configuration that is necessary for opening avro files
+#[derive(Clone, Default)]
+pub struct AvroSource {
+    schema: Option<SchemaRef>,
+    batch_size: Option<usize>,
+    projection: Option<Vec<String>>,
+    metrics: ExecutionPlanMetricsSet,
+    projected_statistics: Option<Statistics>,
+}
+
+impl AvroSource {
+    /// Initialize an AvroSource with default values
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    #[cfg(feature = "avro")]
+    fn open<R: std::io::Read>(&self, reader: R) -> Result<AvroReader<'static, R>> {
+        AvroReader::try_new(
+            reader,
+            Arc::clone(self.schema.as_ref().expect("Schema must set before open")),
+            self.batch_size.expect("Batch size must set before open"),
+            self.projection.clone(),
+        )
+    }
+}
+
+impl FileSource for AvroSource {
+    #[cfg(feature = "avro")]
+    fn create_file_opener(
+        &self,
+        object_store: Arc<dyn ObjectStore>,
+        _base_config: &FileScanConfig,
+        _partition: usize,
+    ) -> Arc<dyn FileOpener> {
+        Arc::new(private::AvroOpener {
+            config: Arc::new(self.clone()),
+            object_store,
+        })
+    }
+
+    #[cfg(not(feature = "avro"))]
+    fn create_file_opener(
+        &self,
+        _object_store: Arc<dyn ObjectStore>,
+        _base_config: &FileScanConfig,
+        _partition: usize,
+    ) -> Arc<dyn FileOpener> {
+        panic!("Avro feature is not enabled in this build")
+    }
+
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
+    fn with_batch_size(&self, batch_size: usize) -> Arc<dyn FileSource> {
+        let mut conf = self.clone();
+        conf.batch_size = Some(batch_size);
+        Arc::new(conf)
+    }
+
+    fn with_schema(&self, schema: SchemaRef) -> Arc<dyn FileSource> {
+        let mut conf = self.clone();
+        conf.schema = Some(schema);
+        Arc::new(conf)
+    }
+    fn with_statistics(&self, statistics: Statistics) -> Arc<dyn FileSource> {
+        let mut conf = self.clone();
+        conf.projected_statistics = Some(statistics);
+        Arc::new(conf)
+    }
+
+    fn with_projection(&self, config: &FileScanConfig) -> Arc<dyn FileSource> {
+        let mut conf = self.clone();
+        conf.projection = config.projected_file_column_names();
+        Arc::new(conf)
+    }
+
+    fn metrics(&self) -> &ExecutionPlanMetricsSet {
+        &self.metrics
+    }
+
+    fn statistics(&self) -> Result<Statistics> {
+        let statistics = &self.projected_statistics;
+        Ok(statistics
+            .clone()
+            .expect("projected_statistics must be set"))
+    }
+
+    fn file_type(&self) -> &str {
+        "avro"
+    }
+
+    fn repartitioned(
+        &self,
+        _target_partitions: usize,
+        _repartition_file_min_size: usize,
+        _output_ordering: Option<LexOrdering>,
+        _config: &FileScanConfig,
+    ) -> Result<Option<FileScanConfig>> {
+        Ok(None)
     }
 }
 
 #[cfg(feature = "avro")]
 mod private {
     use super::*;
-    use crate::datasource::avro_to_arrow::Reader as AvroReader;
-    use crate::datasource::physical_plan::file_stream::{FileOpenFuture, FileOpener};
     use crate::datasource::physical_plan::FileMeta;
+    use crate::datasource::physical_plan::{FileOpenFuture, FileOpener};
 
     use bytes::Buf;
     use futures::StreamExt;
     use object_store::{GetResultPayload, ObjectStore};
 
-    pub struct AvroConfig {
+    pub struct DeprecatedAvroConfig {
         pub schema: SchemaRef,
         pub batch_size: usize,
         pub projection: Option<Vec<String>>,
         pub object_store: Arc<dyn ObjectStore>,
     }
 
-    impl AvroConfig {
+    impl DeprecatedAvroConfig {
         fn open<R: std::io::Read>(&self, reader: R) -> Result<AvroReader<'static, R>> {
             AvroReader::try_new(
                 reader,
@@ -213,15 +297,40 @@ mod private {
         }
     }
 
+    pub struct DeprecatedAvroOpener {
+        pub config: Arc<DeprecatedAvroConfig>,
+    }
+    impl FileOpener for DeprecatedAvroOpener {
+        fn open(&self, file_meta: FileMeta) -> Result<FileOpenFuture> {
+            let config = Arc::clone(&self.config);
+            Ok(Box::pin(async move {
+                let r = config.object_store.get(file_meta.location()).await?;
+                match r.payload {
+                    GetResultPayload::File(file, _) => {
+                        let reader = config.open(file)?;
+                        Ok(futures::stream::iter(reader).boxed())
+                    }
+                    GetResultPayload::Stream(_) => {
+                        let bytes = r.bytes().await?;
+                        let reader = config.open(bytes.reader())?;
+                        Ok(futures::stream::iter(reader).boxed())
+                    }
+                }
+            }))
+        }
+    }
+
     pub struct AvroOpener {
-        pub config: Arc<AvroConfig>,
+        pub config: Arc<AvroSource>,
+        pub object_store: Arc<dyn ObjectStore>,
     }
 
     impl FileOpener for AvroOpener {
         fn open(&self, file_meta: FileMeta) -> Result<FileOpenFuture> {
             let config = Arc::clone(&self.config);
+            let object_store = Arc::clone(&self.object_store);
             Ok(Box::pin(async move {
-                let r = config.object_store.get(file_meta.location()).await?;
+                let r = object_store.get(file_meta.location()).await?;
                 match r.payload {
                     GetResultPayload::File(file, _) => {
                         let reader = config.open(file)?;
@@ -250,6 +359,8 @@ mod tests {
     use crate::scalar::ScalarValue;
     use crate::test::object_store::local_unpartitioned_file;
 
+    use datafusion_physical_plan::ExecutionPlan;
+
     use futures::StreamExt;
     use object_store::chunked::ChunkedStore;
     use object_store::local::LocalFileSystem;
@@ -289,19 +400,21 @@ mod tests {
             .infer_schema(&state, &store, std::slice::from_ref(&meta))
             .await?;
 
-        let avro_exec = AvroExec::new(
-            FileScanConfig::new(ObjectStoreUrl::local_filesystem(), file_schema)
+        let source = Arc::new(AvroSource::new());
+        let conf =
+            FileScanConfig::new(ObjectStoreUrl::local_filesystem(), file_schema, source)
                 .with_file(meta.into())
-                .with_projection(Some(vec![0, 1, 2])),
-        );
+                .with_projection(Some(vec![0, 1, 2]));
+
+        let source_exec = conf.build();
         assert_eq!(
-            avro_exec
+            source_exec
                 .properties()
                 .output_partitioning()
                 .partition_count(),
             1
         );
-        let mut results = avro_exec
+        let mut results = source_exec
             .execute(0, state.task_ctx())
             .expect("plan execution failed");
 
@@ -361,20 +474,21 @@ mod tests {
         // Include the missing column in the projection
         let projection = Some(vec![0, 1, 2, actual_schema.fields().len()]);
 
-        let avro_exec = AvroExec::new(
-            FileScanConfig::new(object_store_url, file_schema)
-                .with_file(meta.into())
-                .with_projection(projection),
-        );
+        let source = Arc::new(AvroSource::new());
+        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+            .with_file(meta.into())
+            .with_projection(projection);
+
+        let source_exec = conf.build();
         assert_eq!(
-            avro_exec
+            source_exec
                 .properties()
                 .output_partitioning()
                 .partition_count(),
             1
         );
 
-        let mut results = avro_exec
+        let mut results = source_exec
             .execute(0, state.task_ctx())
             .expect("plan execution failed");
 
@@ -431,27 +545,25 @@ mod tests {
         partitioned_file.partition_values = vec![ScalarValue::from("2021-10-26")];
 
         let projection = Some(vec![0, 1, file_schema.fields().len(), 2]);
-        let avro_exec = AvroExec::new(
-            FileScanConfig::new(object_store_url, file_schema)
-                // select specific columns of the files as well as the partitioning
-                // column which is supposed to be the last column in the table schema.
-                .with_projection(projection)
-                .with_file(partitioned_file)
-                .with_table_partition_cols(vec![Field::new(
-                    "date",
-                    DataType::Utf8,
-                    false,
-                )]),
-        );
+        let source = Arc::new(AvroSource::new());
+        let conf = FileScanConfig::new(object_store_url, file_schema, source)
+            // select specific columns of the files as well as the partitioning
+            // column which is supposed to be the last column in the table schema.
+            .with_projection(projection)
+            .with_file(partitioned_file)
+            .with_table_partition_cols(vec![Field::new("date", DataType::Utf8, false)]);
+
+        let source_exec = conf.build();
+
         assert_eq!(
-            avro_exec
+            source_exec
                 .properties()
                 .output_partitioning()
                 .partition_count(),
             1
         );
 
-        let mut results = avro_exec
+        let mut results = source_exec
             .execute(0, state.task_ctx())
             .expect("plan execution failed");
 
