@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::sync::Arc;
+
 use crate::parquet::Unit::Page;
 use crate::parquet::{ContextWithParquet, Scenario};
 
@@ -22,7 +24,8 @@ use datafusion::datasource::file_format::parquet::ParquetFormat;
 use datafusion::datasource::file_format::FileFormat;
 use datafusion::datasource::listing::PartitionedFile;
 use datafusion::datasource::object_store::ObjectStoreUrl;
-use datafusion::datasource::physical_plan::{FileScanConfig, ParquetExec};
+use datafusion::datasource::physical_plan::{FileScanConfig, ParquetSource};
+use datafusion::datasource::source::DataSourceExec;
 use datafusion::execution::context::SessionState;
 use datafusion::physical_plan::metrics::MetricValue;
 use datafusion::physical_plan::ExecutionPlan;
@@ -36,7 +39,7 @@ use futures::StreamExt;
 use object_store::path::Path;
 use object_store::ObjectMeta;
 
-async fn get_parquet_exec(state: &SessionState, filter: Expr) -> ParquetExec {
+async fn get_parquet_exec(state: &SessionState, filter: Expr) -> DataSourceExec {
     let object_store_url = ObjectStoreUrl::local_filesystem();
     let store = state.runtime_env().object_store(&object_store_url).unwrap();
 
@@ -71,12 +74,15 @@ async fn get_parquet_exec(state: &SessionState, filter: Expr) -> ParquetExec {
     let execution_props = ExecutionProps::new();
     let predicate = create_physical_expr(&filter, &df_schema, &execution_props).unwrap();
 
-    ParquetExec::builder(
-        FileScanConfig::new(object_store_url, schema).with_file(partitioned_file),
-    )
-    .with_predicate(predicate)
-    .build()
-    .with_enable_page_index(true)
+    let source = Arc::new(
+        ParquetSource::default()
+            .with_predicate(Arc::clone(&schema), predicate)
+            .with_enable_page_index(true),
+    );
+    let base_config =
+        FileScanConfig::new(object_store_url, schema, source).with_file(partitioned_file);
+
+    DataSourceExec::new(Arc::new(base_config))
 }
 
 #[tokio::test]
