@@ -31,17 +31,17 @@ use datafusion_common::ScalarValue;
 use datafusion_common::{
     downcast_value, internal_err, not_impl_err, DataFusionError, Result,
 };
-use datafusion_expr::aggregate_doc_sections::DOC_SECTION_APPROXIMATE;
 use datafusion_expr::function::{AccumulatorArgs, StateFieldsArgs};
 use datafusion_expr::utils::format_state_name;
 use datafusion_expr::{
     Accumulator, AggregateUDFImpl, Documentation, Signature, Volatility,
 };
+use datafusion_macros::user_doc;
 use std::any::Any;
 use std::fmt::{Debug, Formatter};
 use std::hash::Hash;
 use std::marker::PhantomData;
-use std::sync::OnceLock;
+
 make_udaf_expr_and_func!(
     ApproxDistinct,
     approx_distinct,
@@ -243,6 +243,20 @@ impl Default for ApproxDistinct {
     }
 }
 
+#[user_doc(
+    doc_section(label = "Approximate Functions"),
+    description = "Returns the approximate number of distinct input values calculated using the HyperLogLog algorithm.",
+    syntax_example = "approx_distinct(expression)",
+    sql_example = r#"```sql
+> SELECT approx_distinct(column_name) FROM table_name;
++-----------------------------------+
+| approx_distinct(column_name)      |
++-----------------------------------+
+| 42                                |
++-----------------------------------+
+```"#,
+    standard_argument(name = "expression",)
+)]
 pub struct ApproxDistinct {
     signature: Signature,
 }
@@ -309,31 +323,6 @@ impl AggregateUDFImpl for ApproxDistinct {
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_approx_distinct_doc())
+        self.doc()
     }
 }
-
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_approx_distinct_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder()
-            .with_doc_section(DOC_SECTION_APPROXIMATE)
-            .with_description(
-                "Returns the approximate number of distinct input values calculated using the HyperLogLog algorithm.",
-            )
-            .with_syntax_example("approx_distinct(expression)")
-            .with_sql_example(r#"```sql
-> SELECT approx_distinct(column_name) FROM table_name;
-+-----------------------------------+
-| approx_distinct(column_name)      |
-+-----------------------------------+
-| 42                                |
-+-----------------------------------+
-```"#, 
-            )
-            .with_standard_argument("expression", None)
-            .build()
-            .unwrap()
-    })
-}
