@@ -109,8 +109,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             format!("substr_string_view [size={}, strlen={}]", size, len),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -120,8 +120,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             format!("substr_string [size={}, strlen={}]", size, len),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -131,8 +131,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             format!("substr_large_string [size={}, strlen={}]", size, len),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -154,8 +154,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -168,8 +168,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -182,8 +182,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -205,8 +205,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -219,8 +219,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
@@ -233,8 +233,8 @@ fn criterion_benchmark(c: &mut Criterion) {
             ),
             |b| {
                 b.iter(|| {
-                    #[allow(deprecated)] // TODO use invoke_batch
-                    black_box(substr.invoke(&args))
+                    // TODO use invoke_with_args
+                    black_box(substr.invoke_batch(&args, size))
                 })
             },
         );
