@@ -19,15 +19,14 @@
 
 mod required_indices;
 
-use std::collections::{HashMap, HashSet};
-use std::sync::Arc;
-
 use crate::optimizer::ApplyOrder;
 use crate::{OptimizerConfig, OptimizerRule};
+use std::collections::HashSet;
+use std::sync::Arc;
 
 use datafusion_common::{
     get_required_group_by_exprs_indices, internal_datafusion_err, internal_err, Column,
-    JoinType, Result,
+    HashMap, JoinType, Result,
 };
 use datafusion_expr::expr::Alias;
 use datafusion_expr::Unnest;
@@ -36,10 +35,10 @@ use datafusion_expr::{
     TableScan, Window,
 };
 
-use crate::optimize_projections::required_indices::RequiredIndicies;
+use crate::optimize_projections::required_indices::RequiredIndices;
 use crate::utils::NamePreserver;
 use datafusion_common::tree_node::{
-    Transformed, TreeNode, TreeNodeIterator, TreeNodeRecursion,
+    Transformed, TreeNode, TreeNodeContainer, TreeNodeRecursion,
 };
 
 /// Optimizer rule to prune unnecessary columns from intermediate schemas
@@ -86,7 +85,7 @@ impl OptimizerRule for OptimizeProjections {
         config: &dyn OptimizerConfig,
     ) -> Result<Transformed<LogicalPlan>> {
         // All output fields are necessary:
-        let indices = RequiredIndicies::new_for_all_exprs(&plan);
+        let indices = RequiredIndices::new_for_all_exprs(&plan);
         optimize_projections(plan, config, indices)
     }
 }
@@ -110,10 +109,11 @@ impl OptimizerRule for OptimizeProjections {
 ///   columns.
 /// - `Ok(None)`: Signal that the given logical plan did not require any change.
 /// - `Err(error)`: An error occurred during the optimization process.
+#[cfg_attr(feature = "recursive_protection", recursive::recursive)]
 fn optimize_projections(
     plan: LogicalPlan,
     config: &dyn OptimizerConfig,
-    indices: RequiredIndicies,
+    indices: RequiredIndices,
 ) -> Result<Transformed<LogicalPlan>> {
     // Recursively rewrite any nodes that may be able to avoid computation given
     // their parents' required indices.
@@ -176,7 +176,7 @@ fn optimize_projections(
             let all_exprs_iter = new_group_bys.iter().chain(new_aggr_expr.iter());
             let schema = aggregate.input.schema();
             let necessary_indices =
-                RequiredIndicies::new().with_exprs(schema, all_exprs_iter);
+                RequiredIndices::new().with_exprs(schema, all_exprs_iter);
             let necessary_exprs = necessary_indices.get_required_exprs(schema);
 
             return optimize_projections(
@@ -274,7 +274,7 @@ fn optimize_projections(
 
     // For other plan node types, calculate indices for columns they use and
     // try to rewrite their children
-    let mut child_required_indices: Vec<RequiredIndicies> = match &plan {
+    let mut child_required_indices: Vec<RequiredIndices> = match &plan {
         LogicalPlan::Sort(_)
         | LogicalPlan::Filter(_)
         | LogicalPlan::Repartition(_)
@@ -295,7 +295,7 @@ fn optimize_projections(
                 })
                 .collect::<Result<_>>()?
         }
-        LogicalPlan::Limit(_) | LogicalPlan::Prepare(_) => {
+        LogicalPlan::Limit(_) => {
             // Pass index requirements from the parent as well as column indices
             // that appear in this plan's expressions to its child. These operators
             // do not benefit from "small" inputs, so the projection_beneficial
@@ -311,6 +311,7 @@ fn optimize_projections(
         | LogicalPlan::Explain(_)
         | LogicalPlan::Analyze(_)
         | LogicalPlan::Subquery(_)
+        | LogicalPlan::Statement(_)
         | LogicalPlan::Distinct(Distinct::All(_)) => {
             // These plans require all their fields, and their children should
             // be treated as final plans -- otherwise, we may have schema a
@@ -319,7 +320,7 @@ fn optimize_projections(
             //       EXISTS expression), we may not need to require all indices.
             plan.inputs()
                 .into_iter()
-                .map(RequiredIndicies::new_for_all_exprs)
+                .map(RequiredIndices::new_for_all_exprs)
                 .collect()
         }
         LogicalPlan::Extension(extension) => {
@@ -339,17 +340,15 @@ fn optimize_projections(
                 .into_iter()
                 .zip(necessary_children_indices)
                 .map(|(child, necessary_indices)| {
-                    RequiredIndicies::new_from_indices(necessary_indices)
+                    RequiredIndices::new_from_indices(necessary_indices)
                         .with_plan_exprs(&plan, child.schema())
                 })
                 .collect::<Result<Vec<_>>>()?
         }
         LogicalPlan::EmptyRelation(_)
         | LogicalPlan::RecursiveQuery(_)
-        | LogicalPlan::Statement(_)
         | LogicalPlan::Values(_)
-        | LogicalPlan::DescribeTable(_)
-        | LogicalPlan::Execute(_) => {
+        | LogicalPlan::DescribeTable(_) => {
             // These operators have no inputs, so stop the optimization process.
             return Ok(Transformed::no(plan));
         }
@@ -380,7 +379,7 @@ fn optimize_projections(
         LogicalPlan::Unnest(Unnest {
             dependency_indices, ..
         }) => {
-            vec![RequiredIndicies::new_from_indices(
+            vec![RequiredIndices::new_from_indices(
                 dependency_indices.clone(),
             )]
         }
@@ -444,7 +443,7 @@ fn optimize_projections(
 /// - `Ok(Some(Projection))`: Merge was beneficial and successful. Contains the
 ///   merged projection.
 /// - `Ok(None)`: Signals that merge is not beneficial (and has not taken place).
-/// - `Err(error)`: An error occured during the function call.
+/// - `Err(error)`: An error occurred during the function call.
 fn merge_consecutive_projections(proj: Projection) -> Result<Transformed<Projection>> {
     let Projection {
         expr,
@@ -484,7 +483,7 @@ fn merge_consecutive_projections(proj: Projection) -> Result<Transformed<Project
     // previous projection as input:
     let name_preserver = NamePreserver::new_for_projection();
     let mut original_names = vec![];
-    let new_exprs = expr.into_iter().map_until_stop_and_collect(|expr| {
+    let new_exprs = expr.map_elements(|expr| {
         original_names.push(name_preserver.save(&expr));
 
         // do not rewrite top level Aliases (rewriter will remove all aliases within exprs)
@@ -673,9 +672,9 @@ fn outer_columns_helper_multi<'a, 'b>(
 /// adjusted based on the join type.
 fn split_join_requirements(
     left_len: usize,
-    indices: RequiredIndicies,
+    indices: RequiredIndices,
     join_type: &JoinType,
-) -> (RequiredIndicies, RequiredIndicies) {
+) -> (RequiredIndices, RequiredIndices) {
     match join_type {
         // In these cases requirements are split between left/right children:
         JoinType::Inner
@@ -688,10 +687,10 @@ fn split_join_requirements(
             indices.split_off(left_len)
         }
         // All requirements can be re-routed to left child directly.
-        JoinType::LeftAnti | JoinType::LeftSemi => (indices, RequiredIndicies::new()),
+        JoinType::LeftAnti | JoinType::LeftSemi => (indices, RequiredIndices::new()),
         // All requirements can be re-routed to right side directly.
         // No need to change index, join schema is right child schema.
-        JoinType::RightSemi | JoinType::RightAnti => (RequiredIndicies::new(), indices),
+        JoinType::RightSemi | JoinType::RightAnti => (RequiredIndices::new(), indices),
     }
 }
 
@@ -742,18 +741,18 @@ fn add_projection_on_top_if_helpful(
 ///
 /// - `Ok(Some(LogicalPlan))`: Contains the rewritten projection
 /// - `Ok(None)`: No rewrite necessary.
-/// - `Err(error)`: An error occured during the function call.
+/// - `Err(error)`: An error occurred during the function call.
 fn rewrite_projection_given_requirements(
     proj: Projection,
     config: &dyn OptimizerConfig,
-    indices: &RequiredIndicies,
+    indices: &RequiredIndices,
 ) -> Result<Transformed<LogicalPlan>> {
     let Projection { expr, input, .. } = proj;
 
     let exprs_used = indices.get_at_indices(&expr);
 
     let required_indices =
-        RequiredIndicies::new().with_exprs(input.schema(), exprs_used.iter());
+        RequiredIndices::new().with_exprs(input.schema(), exprs_used.iter());
 
     // rewrite the children projection, and if they are changed rewrite the
     // projection down
