@@ -172,13 +172,11 @@ impl PhysicalSortExpr {
         let nullable = self.expr.nullable(schema).unwrap_or(true);
         self.expr.eq(&requirement.expr)
             && if nullable {
-                requirement
-                    .options
-                    .map_or(true, |opts| self.options == opts)
+                requirement.options.is_none_or(|opts| self.options == opts)
             } else {
                 requirement
                     .options
-                    .map_or(true, |opts| self.options.descending == opts.descending)
+                    .is_none_or(|opts| self.options.descending == opts.descending)
             }
     }
 }
@@ -291,9 +289,9 @@ impl PhysicalSortRequirement {
     /// Returns whether this requirement is equal or more specific than `other`.
     pub fn compatible(&self, other: &PhysicalSortRequirement) -> bool {
         self.expr.eq(&other.expr)
-            && other.options.map_or(true, |other_opts| {
-                self.options.map_or(false, |opts| opts == other_opts)
-            })
+            && other
+                .options
+                .is_none_or(|other_opts| self.options == Some(other_opts))
     }
 
     #[deprecated(since = "43.0.0", note = "use  LexRequirement::from_lex_ordering")]
@@ -325,9 +323,13 @@ fn to_str(options: &SortOptions) -> &str {
 
 ///`LexOrdering` contains a `Vec<PhysicalSortExpr>`, which represents
 /// a lexicographical ordering.
+///
+/// For example, `vec![a ASC, b DESC]` represents a lexicographical ordering
+/// that first sorts by column `a` in ascending order, then by column `b` in
+/// descending order.
 #[derive(Debug, Default, Clone, PartialEq, Eq, Hash)]
 pub struct LexOrdering {
-    pub inner: Vec<PhysicalSortExpr>,
+    inner: Vec<PhysicalSortExpr>,
 }
 
 impl AsRef<LexOrdering> for LexOrdering {
@@ -337,7 +339,7 @@ impl AsRef<LexOrdering> for LexOrdering {
 }
 
 impl LexOrdering {
-    // Creates a new [`LexOrdering`] from a vector
+    /// Creates a new [`LexOrdering`] from a vector
     pub fn new(inner: Vec<PhysicalSortExpr>) -> Self {
         Self { inner }
     }
@@ -348,46 +350,66 @@ impl LexOrdering {
         &EMPTY_ORDER
     }
 
+    /// Returns the number of elements that can be stored in the LexOrdering
+    /// without reallocating.
     pub fn capacity(&self) -> usize {
         self.inner.capacity()
     }
 
+    /// Clears the LexOrdering, removing all elements.
     pub fn clear(&mut self) {
         self.inner.clear()
     }
 
+    /// Takes ownership of the actual vector of `PhysicalSortExpr`s in the LexOrdering.
+    pub fn take_exprs(self) -> Vec<PhysicalSortExpr> {
+        self.inner
+    }
+
+    /// Returns `true` if the LexOrdering contains `expr`
     pub fn contains(&self, expr: &PhysicalSortExpr) -> bool {
         self.inner.contains(expr)
     }
 
+    /// Add all elements from `iter` to the LexOrdering.
     pub fn extend<I: IntoIterator<Item = PhysicalSortExpr>>(&mut self, iter: I) {
         self.inner.extend(iter)
     }
 
+    /// Remove all elements from the LexOrdering where `f` evaluates to `false`.
+    pub fn retain<F>(&mut self, f: F)
+    where
+        F: FnMut(&PhysicalSortExpr) -> bool,
+    {
+        self.inner.retain(f)
+    }
+
+    /// Returns `true` if the LexOrdering contains no elements.
     pub fn is_empty(&self) -> bool {
         self.inner.is_empty()
     }
 
-    pub fn iter(&self) -> impl Iterator<Item = &PhysicalSortExpr> {
+    /// Returns an iterator over each `&PhysicalSortExpr` in the LexOrdering.
+    pub fn iter(&self) -> core::slice::Iter<PhysicalSortExpr> {
         self.inner.iter()
     }
 
+    /// Returns the number of elements in the LexOrdering.
     pub fn len(&self) -> usize {
         self.inner.len()
     }
 
+    /// Removes the last element from the LexOrdering and returns it, or `None` if it is empty.
     pub fn pop(&mut self) -> Option<PhysicalSortExpr> {
         self.inner.pop()
     }
 
+    /// Appends an element to the back of the LexOrdering.
     pub fn push(&mut self, physical_sort_expr: PhysicalSortExpr) {
         self.inner.push(physical_sort_expr)
     }
 
-    pub fn retain(&mut self, f: impl FnMut(&PhysicalSortExpr) -> bool) {
-        self.inner.retain(f)
-    }
-
+    /// Truncates the LexOrdering, keeping only the first `len` elements.
     pub fn truncate(&mut self, len: usize) {
         self.inner.truncate(len)
     }
@@ -400,15 +422,43 @@ impl LexOrdering {
 
     /// Converts a `LexRequirement` into a `LexOrdering`.
     ///
-    /// This function converts `PhysicalSortRequirement` to `PhysicalSortExpr`
-    /// for each entry in the input. If required ordering is None for an entry
-    /// default ordering `ASC, NULLS LAST` if given (see the `PhysicalSortExpr::from`).
+    /// This function converts [`PhysicalSortRequirement`] to [`PhysicalSortExpr`]
+    /// for each entry in the input.
+    ///
+    /// If the required ordering is `None` for an entry in `requirement`, the
+    /// default ordering `ASC, NULLS LAST` is used (see
+    /// [`PhysicalSortExpr::from`]).
     pub fn from_lex_requirement(requirement: LexRequirement) -> LexOrdering {
         requirement
             .into_iter()
             .map(PhysicalSortExpr::from)
             .collect()
     }
+
+    /// Collapse a `LexOrdering` into a new duplicate-free `LexOrdering` based on expression.
+    ///
+    /// This function filters  duplicate entries that have same physical
+    /// expression inside, ignoring [`SortOptions`]. For example:
+    ///
+    /// `vec![a ASC, a DESC]` collapses to `vec![a ASC]`.
+    pub fn collapse(self) -> Self {
+        let mut output = LexOrdering::default();
+        for item in self {
+            if !output.iter().any(|req| req.expr.eq(&item.expr)) {
+                output.push(item);
+            }
+        }
+        output
+    }
+
+    /// Transforms each `PhysicalSortExpr` in the `LexOrdering`
+    /// in place using the provided closure `f`.
+    pub fn transform<F>(&mut self, f: F)
+    where
+        F: FnMut(&mut PhysicalSortExpr),
+    {
+        self.inner.iter_mut().for_each(f);
+    }
 }
 
 impl From<Vec<PhysicalSortExpr>> for LexOrdering {
@@ -423,6 +473,13 @@ impl From<LexRequirement> for LexOrdering {
     }
 }
 
+/// Convert a `LexOrdering` into a `Arc[<PhysicalSortExpr>]` for fast copies
+impl From<LexOrdering> for Arc<[PhysicalSortExpr]> {
+    fn from(value: LexOrdering) -> Self {
+        value.inner.into()
+    }
+}
+
 impl Deref for LexOrdering {
     type Target = [PhysicalSortExpr];
 
@@ -540,6 +597,21 @@ impl LexRequirement {
                 .collect(),
         )
     }
+
+    /// Constructs a duplicate-free `LexOrderingReq` by filtering out
+    /// duplicate entries that have same physical expression inside.
+    ///
+    /// For example, `vec![a Some(ASC), a Some(DESC)]` collapses to `vec![a
+    /// Some(ASC)]`.
+    pub fn collapse(self) -> Self {
+        let mut output = Vec::<PhysicalSortRequirement>::new();
+        for item in self {
+            if !output.iter().any(|req| req.expr.eq(&item.expr)) {
+                output.push(item);
+            }
+        }
+        LexRequirement::new(output)
+    }
 }
 
 impl From<LexOrdering> for LexRequirement {
