@@ -16,6 +16,7 @@
 // under the License.
 
 use datafusion_expr::EmitTo;
+use std::mem::size_of;
 
 /// Tracks grouping state when the data is ordered entirely by its
 /// group keys
@@ -139,7 +140,7 @@ impl GroupOrderingFull {
     }
 
     pub(crate) fn size(&self) -> usize {
-        std::mem::size_of::<Self>()
+        size_of::<Self>()
     }
 }
 
