@@ -30,10 +30,8 @@ use crate::metrics::MetricsSet;
 use crate::stream::RecordBatchStreamAdapter;
 use crate::ExecutionPlanProperties;
 
-use arrow::datatypes::SchemaRef;
-use arrow::record_batch::RecordBatch;
-use arrow_array::{ArrayRef, UInt64Array};
-use arrow_schema::{DataType, Field, Schema};
+use arrow::array::{ArrayRef, RecordBatch, UInt64Array};
+use arrow::datatypes::{DataType, Field, Schema, SchemaRef};
 use datafusion_common::{internal_err, Result};
 use datafusion_execution::TaskContext;
 use datafusion_physical_expr::{Distribution, EquivalenceProperties};
@@ -57,7 +55,12 @@ pub trait DataSink: DisplayAs + Debug + Send + Sync {
     /// [DataSink].
     ///
     /// See [ExecutionPlan::metrics()] for more details
-    fn metrics(&self) -> Option<MetricsSet>;
+    fn metrics(&self) -> Option<MetricsSet> {
+        None
+    }
+
+    /// Returns the sink schema
+    fn schema(&self) -> &SchemaRef;
 
     // TODO add desired input ordering
     // How does this sink want its input ordered?
@@ -83,8 +86,6 @@ pub struct DataSinkExec {
     input: Arc<dyn ExecutionPlan>,
     /// Sink to which to write
     sink: Arc<dyn DataSink>,
-    /// Schema of the sink for validating the input data
-    sink_schema: SchemaRef,
     /// Schema describing the structure of the output data.
     count_schema: SchemaRef,
     /// Optional required sort order for output data.
@@ -103,7 +104,6 @@ impl DataSinkExec {
     pub fn new(
         input: Arc<dyn ExecutionPlan>,
         sink: Arc<dyn DataSink>,
-        sink_schema: SchemaRef,
         sort_order: Option<LexRequirement>,
     ) -> Self {
         let count_schema = make_count_schema();
@@ -111,7 +111,6 @@ impl DataSinkExec {
         Self {
             input,
             sink,
-            sink_schema,
             count_schema: make_count_schema(),
             sort_order,
             cache,
@@ -209,7 +208,6 @@ impl ExecutionPlan for DataSinkExec {
         Ok(Arc::new(Self::new(
             Arc::clone(&children[0]),
             Arc::clone(&self.sink),
-            Arc::clone(&self.sink_schema),
             self.sort_order.clone(),
         )))
     }
@@ -226,7 +224,7 @@ impl ExecutionPlan for DataSinkExec {
         }
         let data = execute_input_stream(
             Arc::clone(&self.input),
-            Arc::clone(&self.sink_schema),
+            Arc::clone(self.sink.schema()),
             0,
             Arc::clone(&context),
         )?;
