@@ -24,18 +24,24 @@ use std::sync::Arc;
 use std::task::Poll;
 
 use super::utils::{
-    asymmetric_join_output_partitioning, need_produce_result_in_final, BatchSplitter,
-    BatchTransformer, NoopBatchTransformer, StatefulStreamResult,
+    asymmetric_join_output_partitioning, get_final_indices_from_shared_bitmap,
+    need_produce_result_in_final, reorder_output_after_swap, swap_join_projection,
+    BatchSplitter, BatchTransformer, NoopBatchTransformer, StatefulStreamResult,
 };
 use crate::coalesce_partitions::CoalescePartitionsExec;
+use crate::common::can_project;
 use crate::execution_plan::{boundedness_from_children, EmissionType};
 use crate::joins::utils::{
     adjust_indices_by_join_type, apply_join_filter_to_indices, build_batch_from_indices,
     build_join_schema, check_join_is_valid, estimate_join_statistics,
-    get_final_indices_from_bit_map, BuildProbeJoinMetrics, ColumnIndex, JoinFilter,
-    OnceAsync, OnceFut,
+    BuildProbeJoinMetrics, ColumnIndex, JoinFilter, OnceAsync, OnceFut,
 };
+use crate::joins::SharedBitmapBuilder;
 use crate::metrics::{ExecutionPlanMetricsSet, MetricsSet};
+use crate::projection::{
+    try_embed_projection, try_pushdown_through_join, EmbeddedProjection, JoinData,
+    ProjectionExec,
+};
 use crate::{
     handle_state, DisplayAs, DisplayFormatType, Distribution, ExecutionPlan,
     ExecutionPlanProperties, PlanProperties, RecordBatchStream,
@@ -47,18 +53,18 @@ use arrow::compute::concat_batches;
 use arrow::datatypes::{Schema, SchemaRef};
 use arrow::record_batch::RecordBatch;
 use datafusion_common::{
-    exec_datafusion_err, internal_err, JoinSide, Result, Statistics,
+    exec_datafusion_err, internal_err, project_schema, JoinSide, Result, Statistics,
 };
 use datafusion_execution::memory_pool::{MemoryConsumer, MemoryReservation};
 use datafusion_execution::TaskContext;
 use datafusion_expr::JoinType;
-use datafusion_physical_expr::equivalence::join_equivalence_properties;
+use datafusion_physical_expr::equivalence::{
+    join_equivalence_properties, ProjectionMapping,
+};
 
 use futures::{ready, Stream, StreamExt, TryStreamExt};
 use parking_lot::Mutex;
 
-/// Shared bitmap for visited left-side indices
-type SharedBitmapBuilder = Mutex<BooleanBufferBuilder>;
 /// Left (build-side) data
 struct JoinLeftData {
     /// Build-side data collected to single batch
@@ -154,7 +160,7 @@ pub struct NestedLoopJoinExec {
     /// How the join is performed
     pub(crate) join_type: JoinType,
     /// The schema once the join is applied
-    schema: SchemaRef,
+    join_schema: SchemaRef,
     /// Future that consumes left input and buffers it in memory
     ///
     /// This structure is *shared* across all output streams.
@@ -164,6 +170,9 @@ pub struct NestedLoopJoinExec {
     inner_table: OnceAsync<JoinLeftData>,
     /// Information of index and left / right placement of columns
     column_indices: Vec<ColumnIndex>,
+    /// Projection to apply to the output of the join
+    projection: Option<Vec<usize>>,
+
     /// Execution metrics
     metrics: ExecutionPlanMetricsSet,
     /// Cache holding plan properties like equivalences, output partitioning etc.
@@ -177,24 +186,31 @@ impl NestedLoopJoinExec {
         right: Arc<dyn ExecutionPlan>,
         filter: Option<JoinFilter>,
         join_type: &JoinType,
+        projection: Option<Vec<usize>>,
     ) -> Result<Self> {
         let left_schema = left.schema();
         let right_schema = right.schema();
         check_join_is_valid(&left_schema, &right_schema, &[])?;
-        let (schema, column_indices) =
+        let (join_schema, column_indices) =
             build_join_schema(&left_schema, &right_schema, join_type);
-        let schema = Arc::new(schema);
-        let cache =
-            Self::compute_properties(&left, &right, Arc::clone(&schema), *join_type);
+        let join_schema = Arc::new(join_schema);
+        let cache = Self::compute_properties(
+            &left,
+            &right,
+            Arc::clone(&join_schema),
+            *join_type,
+            projection.as_ref(),
+        )?;
 
         Ok(NestedLoopJoinExec {
             left,
             right,
             filter,
             join_type: *join_type,
-            schema,
+            join_schema,
             inner_table: Default::default(),
             column_indices,
+            projection,
             metrics: Default::default(),
             cache,
         })
@@ -220,26 +236,31 @@ impl NestedLoopJoinExec {
         &self.join_type
     }
 
+    pub fn projection(&self) -> Option<&Vec<usize>> {
+        self.projection.as_ref()
+    }
+
     /// This function creates the cache object that stores the plan properties such as schema, equivalence properties, ordering, partitioning, etc.
     fn compute_properties(
         left: &Arc<dyn ExecutionPlan>,
         right: &Arc<dyn ExecutionPlan>,
         schema: SchemaRef,
         join_type: JoinType,
-    ) -> PlanProperties {
+        projection: Option<&Vec<usize>>,
+    ) -> Result<PlanProperties> {
         // Calculate equivalence properties:
-        let eq_properties = join_equivalence_properties(
+        let mut eq_properties = join_equivalence_properties(
             left.equivalence_properties().clone(),
             right.equivalence_properties().clone(),
             &join_type,
-            schema,
+            Arc::clone(&schema),
             &Self::maintains_input_order(join_type),
             None,
             // No on columns in nested loop join
             &[],
         );
 
-        let output_partitioning =
+        let mut output_partitioning =
             asymmetric_join_output_partitioning(left, right, &join_type);
 
         let emission_type = if left.boundedness().is_unbounded() {
@@ -264,12 +285,22 @@ impl NestedLoopJoinExec {
             right.pipeline_behavior()
         };
 
-        PlanProperties::new(
+        if let Some(projection) = projection {
+            // construct a map from the input expressions to the output expression of the Projection
+            let projection_mapping =
+                ProjectionMapping::from_indices(projection, &schema)?;
+            let out_schema = project_schema(&schema, Some(projection))?;
+            output_partitioning =
+                output_partitioning.project(&projection_mapping, &eq_properties);
+            eq_properties = eq_properties.project(&projection_mapping, out_schema);
+        }
+
+        Ok(PlanProperties::new(
             eq_properties,
             output_partitioning,
             emission_type,
             boundedness_from_children([left, right]),
-        )
+        ))
     }
 
     /// Returns a vector indicating whether the left and right inputs maintain their order.
@@ -296,6 +327,69 @@ impl NestedLoopJoinExec {
             ),
         ]
     }
+
+    pub fn contains_projection(&self) -> bool {
+        self.projection.is_some()
+    }
+
+    pub fn with_projection(&self, projection: Option<Vec<usize>>) -> Result<Self> {
+        // check if the projection is valid
+        can_project(&self.schema(), projection.as_ref())?;
+        let projection = match projection {
+            Some(projection) => match &self.projection {
+                Some(p) => Some(projection.iter().map(|i| p[*i]).collect()),
+                None => Some(projection),
+            },
+            None => None,
+        };
+        Self::try_new(
+            Arc::clone(&self.left),
+            Arc::clone(&self.right),
+            self.filter.clone(),
+            &self.join_type,
+            projection,
+        )
+    }
+
+    /// Returns a new `ExecutionPlan` that runs NestedLoopsJoins with the left
+    /// and right inputs swapped.
+    pub fn swap_inputs(&self) -> Result<Arc<dyn ExecutionPlan>> {
+        let left = self.left();
+        let right = self.right();
+        let new_join = NestedLoopJoinExec::try_new(
+            Arc::clone(right),
+            Arc::clone(left),
+            self.filter().map(JoinFilter::swap),
+            &self.join_type().swap(),
+            swap_join_projection(
+                left.schema().fields().len(),
+                right.schema().fields().len(),
+                self.projection.as_ref(),
+                self.join_type(),
+            ),
+        )?;
+
+        // For Semi/Anti joins, swap result will produce same output schema,
+        // no need to wrap them into additional projection
+        let plan: Arc<dyn ExecutionPlan> = if matches!(
+            self.join_type(),
+            JoinType::LeftSemi
+                | JoinType::RightSemi
+                | JoinType::LeftAnti
+                | JoinType::RightAnti
+        ) || self.projection.is_some()
+        {
+            Arc::new(new_join)
+        } else {
+            reorder_output_after_swap(
+                Arc::new(new_join),
+                &self.left().schema(),
+                &self.right().schema(),
+            )?
+        };
+
+        Ok(plan)
+    }
 }
 
 impl DisplayAs for NestedLoopJoinExec {
@@ -306,10 +400,28 @@ impl DisplayAs for NestedLoopJoinExec {
                     || "".to_string(),
                     |f| format!(", filter={}", f.expression()),
                 );
+                let display_projections = if self.contains_projection() {
+                    format!(
+                        ", projection=[{}]",
+                        self.projection
+                            .as_ref()
+                            .unwrap()
+                            .iter()
+                            .map(|index| format!(
+                                "{}@{}",
+                                self.join_schema.fields().get(*index).unwrap().name(),
+                                index
+                            ))
+                            .collect::<Vec<_>>()
+                            .join(", ")
+                    )
+                } else {
+                    "".to_string()
+                };
                 write!(
                     f,
-                    "NestedLoopJoinExec: join_type={:?}{}",
-                    self.join_type, display_filter
+                    "NestedLoopJoinExec: join_type={:?}{}{}",
+                    self.join_type, display_filter, display_projections
                 )
             }
         }
@@ -353,6 +465,7 @@ impl ExecutionPlan for NestedLoopJoinExec {
             Arc::clone(&children[1]),
             self.filter.clone(),
             &self.join_type,
+            self.projection.clone(),
         )?))
     }
 
@@ -391,14 +504,23 @@ impl ExecutionPlan for NestedLoopJoinExec {
         let right_side_ordered =
             self.maintains_input_order()[1] && self.right.output_ordering().is_some();
 
+        // update column indices to reflect the projection
+        let column_indices_after_projection = match &self.projection {
+            Some(projection) => projection
+                .iter()
+                .map(|i| self.column_indices[*i].clone())
+                .collect(),
+            None => self.column_indices.clone(),
+        };
+
         if enforce_batch_size_in_joins {
             Ok(Box::pin(NestedLoopJoinStream {
-                schema: Arc::clone(&self.schema),
+                schema: self.schema(),
                 filter: self.filter.clone(),
                 join_type: self.join_type,
                 outer_table,
                 inner_table,
-                column_indices: self.column_indices.clone(),
+                column_indices: column_indices_after_projection,
                 join_metrics,
                 indices_cache,
                 right_side_ordered,
@@ -408,12 +530,12 @@ impl ExecutionPlan for NestedLoopJoinExec {
             }))
         } else {
             Ok(Box::pin(NestedLoopJoinStream {
-                schema: Arc::clone(&self.schema),
+                schema: self.schema(),
                 filter: self.filter.clone(),
                 join_type: self.join_type,
                 outer_table,
                 inner_table,
-                column_indices: self.column_indices.clone(),
+                column_indices: column_indices_after_projection,
                 join_metrics,
                 indices_cache,
                 right_side_ordered,
@@ -434,9 +556,47 @@ impl ExecutionPlan for NestedLoopJoinExec {
             Arc::clone(&self.right),
             vec![],
             &self.join_type,
-            &self.schema,
+            &self.join_schema,
         )
     }
+
+    /// Tries to push `projection` down through `nested_loop_join`. If possible, performs the
+    /// pushdown and returns a new [`NestedLoopJoinExec`] as the top plan which has projections
+    /// as its children. Otherwise, returns `None`.
+    fn try_swapping_with_projection(
+        &self,
+        projection: &ProjectionExec,
+    ) -> Result<Option<Arc<dyn ExecutionPlan>>> {
+        // TODO: currently if there is projection in NestedLoopJoinExec, we can't push down projection to left or right input. Maybe we can pushdown the mixed projection later.
+        if self.contains_projection() {
+            return Ok(None);
+        }
+
+        if let Some(JoinData {
+            projected_left_child,
+            projected_right_child,
+            join_filter,
+            ..
+        }) = try_pushdown_through_join(
+            projection,
+            self.left(),
+            self.right(),
+            &[],
+            self.schema(),
+            self.filter(),
+        )? {
+            Ok(Some(Arc::new(NestedLoopJoinExec::try_new(
+                Arc::new(projected_left_child),
+                Arc::new(projected_right_child),
+                join_filter,
+                self.join_type(),
+                // Returned early if projection is not None
+                None,
+            )?)))
+        } else {
+            try_embed_projection(projection, self)
+        }
+    }
 }
 
 /// Asynchronously collect input into a single batch, and creates `JoinLeftData` from it
@@ -844,14 +1004,6 @@ fn join_left_and_right_batch(
     )
 }
 
-fn get_final_indices_from_shared_bitmap(
-    shared_bitmap: &SharedBitmapBuilder,
-    join_type: JoinType,
-) -> (UInt64Array, UInt32Array) {
-    let bitmap = shared_bitmap.lock();
-    get_final_indices_from_bit_map(&bitmap, join_type)
-}
-
 impl<T: BatchTransformer + Unpin + Send> Stream for NestedLoopJoinStream<T> {
     type Item = Result<RecordBatch>;
 
@@ -869,17 +1021,23 @@ impl<T: BatchTransformer + Unpin + Send> RecordBatchStream for NestedLoopJoinStr
     }
 }
 
+impl EmbeddedProjection for NestedLoopJoinExec {
+    fn with_projection(&self, projection: Option<Vec<usize>>) -> Result<Self> {
+        self.with_projection(projection)
+    }
+}
+
 #[cfg(test)]
 pub(crate) mod tests {
     use super::*;
+    use crate::test::TestMemoryExec;
     use crate::{
-        common, expressions::Column, memory::MemoryExec, repartition::RepartitionExec,
-        test::build_table_i32,
+        common, expressions::Column, repartition::RepartitionExec, test::build_table_i32,
     };
 
+    use arrow::array::Int32Array;
+    use arrow::compute::SortOptions;
     use arrow::datatypes::{DataType, Field};
-    use arrow_array::Int32Array;
-    use arrow_schema::SortOptions;
     use datafusion_common::{assert_batches_sorted_eq, assert_contains, ScalarValue};
     use datafusion_execution::runtime_env::RuntimeEnvBuilder;
     use datafusion_expr::Operator;
@@ -912,8 +1070,8 @@ pub(crate) mod tests {
             vec![batch]
         };
 
-        let mut exec =
-            MemoryExec::try_new(&[batches], Arc::clone(&schema), None).unwrap();
+        let mut source =
+            TestMemoryExec::try_new(&[batches], Arc::clone(&schema), None).unwrap();
         if !sorted_column_names.is_empty() {
             let mut sort_info = LexOrdering::default();
             for name in sorted_column_names {
@@ -927,10 +1085,10 @@ pub(crate) mod tests {
                 };
                 sort_info.push(sort_expr);
             }
-            exec = exec.try_with_sort_information(vec![sort_info]).unwrap();
+            source = source.try_with_sort_information(vec![sort_info]).unwrap();
         }
 
-        Arc::new(exec)
+        Arc::new(TestMemoryExec::update_cache(Arc::new(source)))
     }
 
     fn build_left_table() -> Arc<dyn ExecutionPlan> {
@@ -994,7 +1152,11 @@ pub(crate) mod tests {
             Arc::new(BinaryExpr::new(left_filter, Operator::And, right_filter))
                 as Arc<dyn PhysicalExpr>;
 
-        JoinFilter::new(filter_expression, column_indices, intermediate_schema)
+        JoinFilter::new(
+            filter_expression,
+            column_indices,
+            Arc::new(intermediate_schema),
+        )
     }
 
     pub(crate) async fn multi_partitioned_join_collect(
@@ -1014,7 +1176,7 @@ pub(crate) mod tests {
 
         // Use the required distribution for nested loop join to test partition data
         let nested_loop_join =
-            NestedLoopJoinExec::try_new(left, right, join_filter, join_type)?;
+            NestedLoopJoinExec::try_new(left, right, join_filter, join_type, None)?;
         let columns = columns(&nested_loop_join.schema());
         let mut batches = vec![];
         for i in 0..partition_count {
@@ -1351,7 +1513,7 @@ pub(crate) mod tests {
 
             assert_contains!(
                 err.to_string(),
-                "External error: Resources exhausted: Additional allocation failed with top memory consumers (across reservations) as: NestedLoopJoinLoad[0]"
+                "Resources exhausted: Additional allocation failed with top memory consumers (across reservations) as: NestedLoopJoinLoad[0]"
             );
         }
 
@@ -1404,7 +1566,11 @@ pub(crate) mod tests {
             Arc::new(BinaryExpr::new(left_filter, Operator::And, right_filter))
                 as Arc<dyn PhysicalExpr>;
 
-        JoinFilter::new(filter_expression, column_indices, intermediate_schema)
+        JoinFilter::new(
+            filter_expression,
+            column_indices,
+            Arc::new(intermediate_schema),
+        )
     }
 
     fn generate_columns(num_columns: usize, num_rows: usize) -> Vec<Vec<i32>> {
@@ -1450,6 +1616,7 @@ pub(crate) mod tests {
             Arc::clone(&right),
             Some(filter),
             &join_type,
+            None,
         )?) as Arc<dyn ExecutionPlan>;
         assert_eq!(nested_loop_join.maintains_input_order(), vec![false, true]);
 
