@@ -47,6 +47,10 @@ use crate::joins::utils::{
     BatchTransformer, ColumnIndex, JoinFilter, JoinHashMapType, JoinOn, JoinOnRef,
     NoopBatchTransformer, StatefulStreamResult,
 };
+use crate::projection::{
+    join_allows_pushdown, join_table_borders, new_join_children,
+    physical_to_column_exprs, update_join_filter, update_join_on, ProjectionExec,
+};
 use crate::{
     joins::StreamJoinPartitionMode,
     metrics::{ExecutionPlanMetricsSet, MetricsSet},
@@ -59,9 +63,8 @@ use arrow::array::{
     UInt64Array,
 };
 use arrow::compute::concat_batches;
-use arrow::datatypes::{Schema, SchemaRef};
+use arrow::datatypes::{ArrowNativeType, Schema, SchemaRef};
 use arrow::record_batch::RecordBatch;
-use arrow_buffer::ArrowNativeType;
 use datafusion_common::hash_utils::create_hashes;
 use datafusion_common::utils::bisect;
 use datafusion_common::{internal_err, plan_err, HashSet, JoinSide, JoinType, Result};
@@ -71,9 +74,9 @@ use datafusion_expr::interval_arithmetic::Interval;
 use datafusion_physical_expr::equivalence::join_equivalence_properties;
 use datafusion_physical_expr::intervals::cp_solver::ExprIntervalGraph;
 use datafusion_physical_expr::PhysicalExprRef;
+use datafusion_physical_expr_common::sort_expr::{LexOrdering, LexRequirement};
 
 use ahash::RandomState;
-use datafusion_physical_expr_common::sort_expr::{LexOrdering, LexRequirement};
 use futures::{ready, Stream, StreamExt};
 use parking_lot::Mutex;
 
@@ -556,6 +559,81 @@ impl ExecutionPlan for SymmetricHashJoinExec {
             }))
         }
     }
+
+    /// Tries to swap the projection with its input [`SymmetricHashJoinExec`]. If it can be done,
+    /// it returns the new swapped version having the [`SymmetricHashJoinExec`] as the top plan.
+    /// Otherwise, it returns None.
+    fn try_swapping_with_projection(
+        &self,
+        projection: &ProjectionExec,
+    ) -> Result<Option<Arc<dyn ExecutionPlan>>> {
+        // Convert projected PhysicalExpr's to columns. If not possible, we cannot proceed.
+        let Some(projection_as_columns) = physical_to_column_exprs(projection.expr())
+        else {
+            return Ok(None);
+        };
+
+        let (far_right_left_col_ind, far_left_right_col_ind) = join_table_borders(
+            self.left().schema().fields().len(),
+            &projection_as_columns,
+        );
+
+        if !join_allows_pushdown(
+            &projection_as_columns,
+            &self.schema(),
+            far_right_left_col_ind,
+            far_left_right_col_ind,
+        ) {
+            return Ok(None);
+        }
+
+        let Some(new_on) = update_join_on(
+            &projection_as_columns[0..=far_right_left_col_ind as _],
+            &projection_as_columns[far_left_right_col_ind as _..],
+            self.on(),
+            self.left().schema().fields().len(),
+        ) else {
+            return Ok(None);
+        };
+
+        let new_filter = if let Some(filter) = self.filter() {
+            match update_join_filter(
+                &projection_as_columns[0..=far_right_left_col_ind as _],
+                &projection_as_columns[far_left_right_col_ind as _..],
+                filter,
+                self.left().schema().fields().len(),
+            ) {
+                Some(updated_filter) => Some(updated_filter),
+                None => return Ok(None),
+            }
+        } else {
+            None
+        };
+
+        let (new_left, new_right) = new_join_children(
+            &projection_as_columns,
+            far_right_left_col_ind,
+            far_left_right_col_ind,
+            self.left(),
+            self.right(),
+        )?;
+
+        Ok(Some(Arc::new(SymmetricHashJoinExec::try_new(
+            Arc::new(new_left),
+            Arc::new(new_right),
+            new_on,
+            new_filter,
+            self.join_type(),
+            self.null_equals_null(),
+            self.right()
+                .output_ordering()
+                .map(|p| LexOrdering::new(p.to_vec())),
+            self.left()
+                .output_ordering()
+                .map(|p| LexOrdering::new(p.to_vec())),
+            self.partition_mode(),
+        )?)))
+    }
 }
 
 /// A stream that issues [RecordBatch]es as they arrive from the right  of the join.
@@ -1632,7 +1710,7 @@ pub enum SHJStreamState {
 #[cfg(test)]
 mod tests {
     use std::collections::HashMap;
-    use std::sync::Mutex;
+    use std::sync::{LazyLock, Mutex};
 
     use super::*;
     use crate::joins::test_utils::{
@@ -1650,7 +1728,6 @@ mod tests {
     use datafusion_physical_expr::expressions::{binary, col, lit, Column};
     use datafusion_physical_expr_common::sort_expr::{LexOrdering, PhysicalSortExpr};
 
-    use once_cell::sync::Lazy;
     use rstest::*;
 
     const TABLE_SIZE: i32 = 30;
@@ -1659,8 +1736,8 @@ mod tests {
     type TableValue = (Vec<RecordBatch>, Vec<RecordBatch>); // (left, right)
 
     // Cache for storing tables
-    static TABLE_CACHE: Lazy<Mutex<HashMap<TableKey, TableValue>>> =
-        Lazy::new(|| Mutex::new(HashMap::new()));
+    static TABLE_CACHE: LazyLock<Mutex<HashMap<TableKey, TableValue>>> =
+        LazyLock::new(|| Mutex::new(HashMap::new()));
 
     fn get_or_create_table(
         cardinality: (i32, i32),
@@ -1799,7 +1876,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -1864,7 +1942,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -1916,7 +1995,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2013,7 +2093,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2071,7 +2152,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
     }
@@ -2129,7 +2211,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2189,7 +2272,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2246,7 +2330,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2310,7 +2395,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
@@ -2394,7 +2480,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
     }
@@ -2468,7 +2555,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
 
         Ok(())
@@ -2539,7 +2627,8 @@ mod tests {
                 side: JoinSide::Right,
             },
         ];
-        let filter = JoinFilter::new(filter_expr, column_indices, intermediate_schema);
+        let filter =
+            JoinFilter::new(filter_expr, column_indices, Arc::new(intermediate_schema));
 
         experiment(left, right, Some(filter), join_type, on, task_ctx).await?;
         Ok(())
