@@ -23,6 +23,7 @@ use std::sync::Arc;
 use crate::common::spawn_buffered;
 use crate::limit::LimitStream;
 use crate::metrics::{BaselineMetrics, ExecutionPlanMetricsSet, MetricsSet};
+use crate::projection::{make_with_child, update_expr, ProjectionExec};
 use crate::sorts::streaming_merge::StreamingMergeBuilder;
 use crate::{
     DisplayAs, DisplayFormatType, Distribution, ExecutionPlan, ExecutionPlanProperties,
@@ -32,8 +33,9 @@ use crate::{
 use datafusion_common::{internal_err, Result};
 use datafusion_execution::memory_pool::MemoryConsumer;
 use datafusion_execution::TaskContext;
-
+use datafusion_physical_expr::PhysicalSortExpr;
 use datafusion_physical_expr_common::sort_expr::{LexOrdering, LexRequirement};
+
 use log::{debug, trace};
 
 /// Sort preserving merge execution plan
@@ -334,6 +336,39 @@ impl ExecutionPlan for SortPreservingMergeExec {
     fn supports_limit_pushdown(&self) -> bool {
         true
     }
+
+    /// Tries to swap the projection with its input [`SortPreservingMergeExec`].
+    /// If this is possible, it returns the new [`SortPreservingMergeExec`] whose
+    /// child is a projection. Otherwise, it returns None.
+    fn try_swapping_with_projection(
+        &self,
+        projection: &ProjectionExec,
+    ) -> Result<Option<Arc<dyn ExecutionPlan>>> {
+        // If the projection does not narrow the schema, we should not try to push it down.
+        if projection.expr().len() >= projection.input().schema().fields().len() {
+            return Ok(None);
+        }
+
+        let mut updated_exprs = LexOrdering::default();
+        for sort in self.expr() {
+            let Some(updated_expr) = update_expr(&sort.expr, projection.expr(), false)?
+            else {
+                return Ok(None);
+            };
+            updated_exprs.push(PhysicalSortExpr {
+                expr: updated_expr,
+                options: sort.options,
+            });
+        }
+
+        Ok(Some(Arc::new(
+            SortPreservingMergeExec::new(
+                updated_exprs,
+                make_with_child(projection, self.input())?,
+            )
+            .with_fetch(self.fetch()),
+        )))
+    }
 }
 
 #[cfg(test)]
@@ -349,21 +384,21 @@ mod tests {
     use crate::coalesce_partitions::CoalescePartitionsExec;
     use crate::execution_plan::{Boundedness, EmissionType};
     use crate::expressions::col;
-    use crate::memory::MemoryExec;
     use crate::metrics::{MetricValue, Timestamp};
     use crate::repartition::RepartitionExec;
     use crate::sorts::sort::SortExec;
     use crate::stream::RecordBatchReceiverStream;
     use crate::test::exec::{assert_strong_count_converges_to_zero, BlockingExec};
+    use crate::test::TestMemoryExec;
     use crate::test::{self, assert_is_pending, make_partition};
     use crate::{collect, common};
 
-    use arrow::array::{ArrayRef, Int32Array, StringArray, TimestampNanosecondArray};
+    use arrow::array::{
+        ArrayRef, Int32Array, Int64Array, RecordBatch, StringArray,
+        TimestampNanosecondArray,
+    };
     use arrow::compute::SortOptions;
-    use arrow::datatypes::{DataType, Field, Schema};
-    use arrow::record_batch::RecordBatch;
-    use arrow_array::Int64Array;
-    use arrow_schema::SchemaRef;
+    use arrow::datatypes::{DataType, Field, Schema, SchemaRef};
     use datafusion_common::{assert_batches_eq, assert_contains, DataFusionError};
     use datafusion_common_runtime::SpawnedTask;
     use datafusion_execution::config::SessionConfig;
@@ -415,9 +450,10 @@ mod tests {
             },
         ]);
 
-        let exec = MemoryExec::try_new(&[rbs], schema, None).unwrap();
-        let repartition_exec =
-            RepartitionExec::try_new(Arc::new(exec), Partitioning::RoundRobinBatch(2))?;
+        let repartition_exec = RepartitionExec::try_new(
+            TestMemoryExec::try_new_exec(&[rbs], schema, None).unwrap(),
+            Partitioning::RoundRobinBatch(2),
+        )?;
         let coalesce_batches_exec =
             CoalesceBatchesExec::new(Arc::new(repartition_exec), target_batch_size);
         let spm = SortPreservingMergeExec::new(sort, Arc::new(coalesce_batches_exec))
@@ -507,9 +543,14 @@ mod tests {
 
         let schema = batch.schema();
         let sort = LexOrdering::default(); // no sort expressions
-        let exec = MemoryExec::try_new(&[vec![batch.clone()], vec![batch]], schema, None)
-            .unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec = TestMemoryExec::try_new_exec(
+            &[vec![batch.clone()], vec![batch]],
+            schema,
+            None,
+        )
+        .unwrap();
+
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let res = collect(merge, task_ctx).await.unwrap_err();
         assert_contains!(
@@ -695,8 +736,8 @@ mod tests {
                 options: Default::default(),
             },
         ]);
-        let exec = MemoryExec::try_new(partitions, schema, None).unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec = TestMemoryExec::try_new_exec(partitions, schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let collected = collect(merge, context).await.unwrap();
         assert_batches_eq!(exp, collected.as_slice());
@@ -803,9 +844,7 @@ mod tests {
         let sorted = basic_sort(csv, sort, context).await;
         let split: Vec<_> = sizes.iter().map(|x| split_batch(&sorted, *x)).collect();
 
-        Ok(Arc::new(
-            MemoryExec::try_new(&split, sorted.schema(), None).unwrap(),
-        ))
+        Ok(TestMemoryExec::try_new_exec(&split, sorted.schema(), None).unwrap())
     }
 
     #[tokio::test]
@@ -933,8 +972,9 @@ mod tests {
                 },
             },
         ]);
-        let exec = MemoryExec::try_new(&[vec![b1], vec![b2]], schema, None).unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec =
+            TestMemoryExec::try_new_exec(&[vec![b1], vec![b2]], schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let collected = collect(merge, task_ctx).await.unwrap();
         assert_eq!(collected.len(), 1);
@@ -975,10 +1015,9 @@ mod tests {
                 nulls_first: true,
             },
         }]);
-        let exec = MemoryExec::try_new(&[vec![batch]], schema, None).unwrap();
-        let merge = Arc::new(
-            SortPreservingMergeExec::new(sort, Arc::new(exec)).with_fetch(Some(2)),
-        );
+        let exec = TestMemoryExec::try_new_exec(&[vec![batch]], schema, None).unwrap();
+        let merge =
+            Arc::new(SortPreservingMergeExec::new(sort, exec).with_fetch(Some(2)));
 
         let collected = collect(merge, task_ctx).await.unwrap();
         assert_eq!(collected.len(), 1);
@@ -1011,8 +1050,8 @@ mod tests {
                 nulls_first: true,
             },
         }]);
-        let exec = MemoryExec::try_new(&[vec![batch]], schema, None).unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec = TestMemoryExec::try_new_exec(&[vec![batch]], schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let collected = collect(merge, task_ctx).await.unwrap();
         assert_eq!(collected.len(), 1);
@@ -1120,8 +1159,9 @@ mod tests {
             expr: col("b", &schema).unwrap(),
             options: Default::default(),
         }]);
-        let exec = MemoryExec::try_new(&[vec![b1], vec![b2]], schema, None).unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec =
+            TestMemoryExec::try_new_exec(&[vec![b1], vec![b2]], schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let collected = collect(Arc::clone(&merge) as Arc<dyn ExecutionPlan>, task_ctx)
             .await
@@ -1231,8 +1271,8 @@ mod tests {
             },
         }]);
 
-        let exec = MemoryExec::try_new(&partitions, schema, None).unwrap();
-        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec)));
+        let exec = TestMemoryExec::try_new_exec(&partitions, schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, exec));
 
         let collected = collect(merge, task_ctx).await.unwrap();
         assert_eq!(collected.len(), 1);
