@@ -30,8 +30,9 @@ use datafusion::datasource::file_format::json::JsonSink;
 #[cfg(feature = "parquet")]
 use datafusion::datasource::file_format::parquet::ParquetSink;
 #[cfg(feature = "parquet")]
-use datafusion::datasource::physical_plan::ParquetExec;
-use datafusion::datasource::physical_plan::{AvroExec, CsvExec};
+use datafusion::datasource::physical_plan::ParquetSource;
+use datafusion::datasource::physical_plan::{AvroSource, CsvSource, FileScanConfig};
+use datafusion::datasource::source::DataSourceExec;
 use datafusion::execution::runtime_env::RuntimeEnv;
 use datafusion::execution::FunctionRegistry;
 use datafusion::physical_expr::aggregate::AggregateFunctionExpr;
@@ -63,6 +64,7 @@ use datafusion::physical_plan::windows::{BoundedWindowAggExec, WindowAggExec};
 use datafusion::physical_plan::{
     ExecutionPlan, InputOrderMode, PhysicalExpr, WindowExpr,
 };
+use datafusion_common::config::TableParquetOptions;
 use datafusion_common::{internal_err, not_impl_err, DataFusionError, Result};
 use datafusion_expr::{AggregateUDF, ScalarUDF, WindowUDF};
 
@@ -70,6 +72,7 @@ use crate::common::{byte_to_string, str_to_byte};
 use crate::physical_plan::from_proto::{
     parse_physical_expr, parse_physical_sort_expr, parse_physical_sort_exprs,
     parse_physical_window_expr, parse_protobuf_file_scan_config,
+    parse_protobuf_file_scan_schema,
 };
 use crate::physical_plan::to_proto::{
     serialize_file_scan_config, serialize_maybe_filter, serialize_physical_aggr_expr,
@@ -203,47 +206,51 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                     )),
                 }
             }
-            PhysicalPlanType::CsvScan(scan) => Ok(Arc::new(
-                CsvExec::builder(parse_protobuf_file_scan_config(
+            PhysicalPlanType::CsvScan(scan) => {
+                let escape = if let Some(
+                    protobuf::csv_scan_exec_node::OptionalEscape::Escape(escape),
+                ) = &scan.optional_escape
+                {
+                    Some(str_to_byte(escape, "escape")?)
+                } else {
+                    None
+                };
+
+                let comment = if let Some(
+                    protobuf::csv_scan_exec_node::OptionalComment::Comment(comment),
+                ) = &scan.optional_comment
+                {
+                    Some(str_to_byte(comment, "comment")?)
+                } else {
+                    None
+                };
+
+                let source = Arc::new(
+                    CsvSource::new(
+                        scan.has_header,
+                        str_to_byte(&scan.delimiter, "delimiter")?,
+                        0,
+                    )
+                    .with_escape(escape)
+                    .with_comment(comment),
+                );
+
+                let conf = parse_protobuf_file_scan_config(
                     scan.base_conf.as_ref().unwrap(),
                     registry,
                     extension_codec,
-                )?)
-                .with_has_header(scan.has_header)
-                .with_delimeter(str_to_byte(&scan.delimiter, "delimiter")?)
-                .with_quote(str_to_byte(&scan.quote, "quote")?)
-                .with_escape(
-                    if let Some(protobuf::csv_scan_exec_node::OptionalEscape::Escape(
-                        escape,
-                    )) = &scan.optional_escape
-                    {
-                        Some(str_to_byte(escape, "escape")?)
-                    } else {
-                        None
-                    },
-                )
-                .with_comment(
-                    if let Some(protobuf::csv_scan_exec_node::OptionalComment::Comment(
-                        comment,
-                    )) = &scan.optional_comment
-                    {
-                        Some(str_to_byte(comment, "comment")?)
-                    } else {
-                        None
-                    },
-                )
+                    source,
+                )?
                 .with_newlines_in_values(scan.newlines_in_values)
-                .with_file_compression_type(FileCompressionType::UNCOMPRESSED)
-                .build(),
-            )),
+                .with_file_compression_type(FileCompressionType::UNCOMPRESSED);
+                Ok(conf.build())
+            }
             #[cfg_attr(not(feature = "parquet"), allow(unused_variables))]
             PhysicalPlanType::ParquetScan(scan) => {
                 #[cfg(feature = "parquet")]
                 {
-                    let base_config = parse_protobuf_file_scan_config(
+                    let schema = parse_protobuf_file_scan_schema(
                         scan.base_conf.as_ref().unwrap(),
-                        registry,
-                        extension_codec,
                     )?;
                     let predicate = scan
                         .predicate
@@ -252,26 +259,40 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             parse_physical_expr(
                                 expr,
                                 registry,
-                                base_config.file_schema.as_ref(),
+                                schema.as_ref(),
                                 extension_codec,
                             )
                         })
                         .transpose()?;
-                    let mut builder = ParquetExec::builder(base_config);
-                    if let Some(predicate) = predicate {
-                        builder = builder.with_predicate(predicate)
+                    let mut options = TableParquetOptions::default();
+
+                    if let Some(table_options) = scan.parquet_options.as_ref() {
+                        options = table_options.try_into()?;
                     }
-                    Ok(builder.build_arc())
+                    let mut source = ParquetSource::new(options);
+
+                    if let Some(predicate) = predicate {
+                        source = source.with_predicate(Arc::clone(&schema), predicate);
+                    }
+                    let base_config = parse_protobuf_file_scan_config(
+                        scan.base_conf.as_ref().unwrap(),
+                        registry,
+                        extension_codec,
+                        Arc::new(source),
+                    )?;
+                    Ok(base_config.build())
                 }
                 #[cfg(not(feature = "parquet"))]
                 panic!("Unable to process a Parquet PhysicalPlan when `parquet` feature is not enabled")
             }
             PhysicalPlanType::AvroScan(scan) => {
-                Ok(Arc::new(AvroExec::new(parse_protobuf_file_scan_config(
+                let conf = parse_protobuf_file_scan_config(
                     scan.base_conf.as_ref().unwrap(),
                     registry,
                     extension_codec,
-                )?)))
+                    Arc::new(AvroSource::new()),
+                )?;
+                Ok(conf.build())
             }
             PhysicalPlanType::CoalesceBatches(coalesce_batches) => {
                 let input: Arc<dyn ExecutionPlan> = into_physical_plan(
@@ -383,14 +404,14 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                     Ok(Arc::new(BoundedWindowAggExec::try_new(
                         physical_window_expr,
                         input,
-                        partition_keys,
                         input_order_mode,
+                        !partition_keys.is_empty(),
                     )?))
                 } else {
                     Ok(Arc::new(WindowAggExec::try_new(
                         physical_window_expr,
                         input,
-                        partition_keys,
+                        !partition_keys.is_empty(),
                     )?))
                 }
             }
@@ -625,7 +646,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             })
                             .collect::<Result<Vec<_>>>()?;
 
-                        Ok(JoinFilter::new(expression, column_indices, schema))
+                        Ok(JoinFilter::new(expression, column_indices, Arc::new(schema)))
                     })
                     .map_or(Ok(None), |v: Result<JoinFilter>| v.map(Some))?;
 
@@ -739,7 +760,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             })
                             .collect::<Result<_>>()?;
 
-                        Ok(JoinFilter::new(expression, column_indices, schema))
+                        Ok(JoinFilter::new(expression, column_indices, Arc::new(schema)))
                     })
                     .map_or(Ok(None), |v: Result<JoinFilter>| v.map(Some))?;
 
@@ -992,15 +1013,27 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             })
                             .collect::<Result<Vec<_>>>()?;
 
-                        Ok(JoinFilter::new(expression, column_indices, schema))
+                        Ok(JoinFilter::new(expression, column_indices, Arc::new(schema)))
                     })
                     .map_or(Ok(None), |v: Result<JoinFilter>| v.map(Some))?;
 
+                let projection = if !join.projection.is_empty() {
+                    Some(
+                        join.projection
+                            .iter()
+                            .map(|i| *i as usize)
+                            .collect::<Vec<_>>(),
+                    )
+                } else {
+                    None
+                };
+
                 Ok(Arc::new(NestedLoopJoinExec::try_new(
                     left,
                     right,
                     filter,
                     &join_type.into(),
+                    projection,
                 )?))
             }
             PhysicalPlanType::Analyze(analyze) => {
@@ -1043,7 +1076,6 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                 Ok(Arc::new(DataSinkExec::new(
                     input,
                     Arc::new(data_sink),
-                    sink_schema,
                     sort_order,
                 )))
             }
@@ -1073,7 +1105,6 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                 Ok(Arc::new(DataSinkExec::new(
                     input,
                     Arc::new(data_sink),
-                    sink_schema,
                     sort_order,
                 )))
             }
@@ -1110,7 +1141,6 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                     Ok(Arc::new(DataSinkExec::new(
                         input,
                         Arc::new(data_sink),
-                        sink_schema,
                         sort_order,
                     )))
                 }
@@ -1307,7 +1337,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             }
                         })
                         .collect();
-                    let schema = f.schema().try_into()?;
+                    let schema = f.schema().as_ref().try_into()?;
                     Ok(protobuf::JoinFilter {
                         expression: Some(expression),
                         column_indices,
@@ -1379,7 +1409,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             }
                         })
                         .collect();
-                    let schema = f.schema().try_into()?;
+                    let schema = f.schema().as_ref().try_into()?;
                     Ok(protobuf::JoinFilter {
                         expression: Some(expression),
                         column_indices,
@@ -1600,67 +1630,101 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
             });
         }
 
-        if let Some(exec) = plan.downcast_ref::<CsvExec>() {
-            return Ok(protobuf::PhysicalPlanNode {
-                physical_plan_type: Some(PhysicalPlanType::CsvScan(
-                    protobuf::CsvScanExecNode {
-                        base_conf: Some(serialize_file_scan_config(
-                            exec.base_config(),
-                            extension_codec,
-                        )?),
-                        has_header: exec.has_header(),
-                        delimiter: byte_to_string(exec.delimiter(), "delimiter")?,
-                        quote: byte_to_string(exec.quote(), "quote")?,
-                        optional_escape: if let Some(escape) = exec.escape() {
-                            Some(protobuf::csv_scan_exec_node::OptionalEscape::Escape(
-                                byte_to_string(escape, "escape")?,
-                            ))
-                        } else {
-                            None
-                        },
-                        optional_comment: if let Some(comment) = exec.comment() {
-                            Some(protobuf::csv_scan_exec_node::OptionalComment::Comment(
-                                byte_to_string(comment, "comment")?,
-                            ))
-                        } else {
-                            None
-                        },
-                        newlines_in_values: exec.newlines_in_values(),
-                    },
-                )),
-            });
+        if let Some(data_source_exec) = plan.downcast_ref::<DataSourceExec>() {
+            let data_source = data_source_exec.data_source();
+            if let Some(maybe_csv) = data_source.as_any().downcast_ref::<FileScanConfig>()
+            {
+                let source = maybe_csv.file_source();
+                if let Some(csv_config) = source.as_any().downcast_ref::<CsvSource>() {
+                    return Ok(protobuf::PhysicalPlanNode {
+                        physical_plan_type: Some(PhysicalPlanType::CsvScan(
+                            protobuf::CsvScanExecNode {
+                                base_conf: Some(serialize_file_scan_config(
+                                    maybe_csv,
+                                    extension_codec,
+                                )?),
+                                has_header: csv_config.has_header(),
+                                delimiter: byte_to_string(
+                                    csv_config.delimiter(),
+                                    "delimiter",
+                                )?,
+                                quote: byte_to_string(csv_config.quote(), "quote")?,
+                                optional_escape: if let Some(escape) = csv_config.escape()
+                                {
+                                    Some(
+                                        protobuf::csv_scan_exec_node::OptionalEscape::Escape(
+                                            byte_to_string(escape, "escape")?,
+                                        ),
+                                    )
+                                } else {
+                                    None
+                                },
+                                optional_comment: if let Some(comment) =
+                                    csv_config.comment()
+                                {
+                                    Some(protobuf::csv_scan_exec_node::OptionalComment::Comment(
+                                        byte_to_string(comment, "comment")?,
+                                    ))
+                                } else {
+                                    None
+                                },
+                                newlines_in_values: maybe_csv.newlines_in_values(),
+                            },
+                        )),
+                    });
+                }
+            }
         }
 
         #[cfg(feature = "parquet")]
-        if let Some(exec) = plan.downcast_ref::<ParquetExec>() {
-            let predicate = exec
-                .predicate()
-                .map(|pred| serialize_physical_expr(pred, extension_codec))
-                .transpose()?;
-            return Ok(protobuf::PhysicalPlanNode {
-                physical_plan_type: Some(PhysicalPlanType::ParquetScan(
-                    protobuf::ParquetScanExecNode {
-                        base_conf: Some(serialize_file_scan_config(
-                            exec.base_config(),
-                            extension_codec,
-                        )?),
-                        predicate,
-                    },
-                )),
-            });
+        if let Some(exec) = plan.downcast_ref::<DataSourceExec>() {
+            let data_source_exec = exec.data_source();
+            if let Some(maybe_parquet) =
+                data_source_exec.as_any().downcast_ref::<FileScanConfig>()
+            {
+                let source = maybe_parquet.file_source();
+                if let Some(conf) = source.as_any().downcast_ref::<ParquetSource>() {
+                    let predicate = conf
+                        .predicate()
+                        .map(|pred| serialize_physical_expr(pred, extension_codec))
+                        .transpose()?;
+                    return Ok(protobuf::PhysicalPlanNode {
+                        physical_plan_type: Some(PhysicalPlanType::ParquetScan(
+                            protobuf::ParquetScanExecNode {
+                                base_conf: Some(serialize_file_scan_config(
+                                    maybe_parquet,
+                                    extension_codec,
+                                )?),
+                                predicate,
+                                parquet_options: Some(
+                                    conf.table_parquet_options().try_into()?,
+                                ),
+                            },
+                        )),
+                    });
+                }
+            }
         }
 
-        if let Some(exec) = plan.downcast_ref::<AvroExec>() {
-            return Ok(protobuf::PhysicalPlanNode {
-                physical_plan_type: Some(PhysicalPlanType::AvroScan(
-                    protobuf::AvroScanExecNode {
-                        base_conf: Some(serialize_file_scan_config(
-                            exec.base_config(),
-                            extension_codec,
-                        )?),
-                    },
-                )),
-            });
+        if let Some(data_source_exec) = plan.downcast_ref::<DataSourceExec>() {
+            let data_source = data_source_exec.data_source();
+            if let Some(maybe_avro) =
+                data_source.as_any().downcast_ref::<FileScanConfig>()
+            {
+                let source = maybe_avro.file_source();
+                if source.as_any().downcast_ref::<AvroSource>().is_some() {
+                    return Ok(protobuf::PhysicalPlanNode {
+                        physical_plan_type: Some(PhysicalPlanType::AvroScan(
+                            protobuf::AvroScanExecNode {
+                                base_conf: Some(serialize_file_scan_config(
+                                    maybe_avro,
+                                    extension_codec,
+                                )?),
+                            },
+                        )),
+                    });
+                }
+            }
         }
 
         if let Some(exec) = plan.downcast_ref::<CoalescePartitionsExec>() {
@@ -1824,7 +1888,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                             }
                         })
                         .collect();
-                    let schema = f.schema().try_into()?;
+                    let schema = f.schema().as_ref().try_into()?;
                     Ok(protobuf::JoinFilter {
                         expression: Some(expression),
                         column_indices,
@@ -1840,6 +1904,9 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                         right: Some(Box::new(right)),
                         join_type: join_type.into(),
                         filter,
+                        projection: exec.projection().map_or_else(Vec::new, |v| {
+                            v.iter().map(|x| *x as u32).collect::<Vec<u32>>()
+                        }),
                     },
                 ))),
             });
@@ -1858,7 +1925,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                 .collect::<Result<Vec<protobuf::PhysicalWindowExprNode>>>()?;
 
             let partition_keys = exec
-                .partition_keys
+                .partition_keys()
                 .iter()
                 .map(|e| serialize_physical_expr(e, extension_codec))
                 .collect::<Result<Vec<protobuf::PhysicalExprNode>>>()?;
@@ -1888,7 +1955,7 @@ impl AsExecutionPlan for protobuf::PhysicalPlanNode {
                 .collect::<Result<Vec<protobuf::PhysicalWindowExprNode>>>()?;
 
             let partition_keys = exec
-                .partition_keys
+                .partition_keys()
                 .iter()
                 .map(|e| serialize_physical_expr(e, extension_codec))
                 .collect::<Result<Vec<protobuf::PhysicalExprNode>>>()?;
