@@ -17,13 +17,13 @@
 
 use crate::planner::{ContextProvider, PlannerContext, SqlToRel};
 
-use arrow_schema::DataType;
+use arrow::datatypes::DataType;
 use datafusion_common::{
     internal_datafusion_err, internal_err, not_impl_err, plan_datafusion_err, plan_err,
-    DFSchema, Dependency, Result,
+    DFSchema, Dependency, Diagnostic, Result, Span,
 };
 use datafusion_expr::expr::{ScalarFunction, Unnest};
-use datafusion_expr::planner::PlannerResult;
+use datafusion_expr::planner::{PlannerResult, RawAggregateExpr, RawWindowExpr};
 use datafusion_expr::{
     expr, qualified_wildcard, wildcard, Expr, ExprFunctionExt, ExprSchemable,
     WindowFrame, WindowFunctionDefinition,
@@ -217,7 +217,7 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
         // it shouldn't have ordering requirement as function argument
         // required ordering should be defined in OVER clause.
         let is_function_window = over.is_some();
-
+        let sql_parser_span = name.0[0].span;
         let name = if name.0.len() > 1 {
             // DF doesn't handle compound identifiers
             // (e.g. "foo.bar") for function names yet
@@ -236,7 +236,6 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
                 }
             }
         }
-
         // User-defined function (UDF) should have precedence
         if let Some(fm) = self.context_provider.get_function_meta(&name) {
             let args = self.function_args_to_expr(args, schema, planner_context)?;
@@ -259,7 +258,6 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
                 "Aggregate ORDER BY is not implemented for window functions"
             );
         }
-
         // Then, window function
         if let Some(WindowType::WindowSpec(window)) = over {
             let partition_by = window
@@ -315,15 +313,38 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
             };
 
             if let Ok(fun) = self.find_window_func(&name) {
-                return Expr::WindowFunction(expr::WindowFunction::new(
-                    fun,
-                    self.function_args_to_expr(args, schema, planner_context)?,
-                ))
-                .partition_by(partition_by)
-                .order_by(order_by)
-                .window_frame(window_frame)
-                .null_treatment(null_treatment)
-                .build();
+                let args = self.function_args_to_expr(args, schema, planner_context)?;
+                let mut window_expr = RawWindowExpr {
+                    func_def: fun,
+                    args,
+                    partition_by,
+                    order_by,
+                    window_frame,
+                    null_treatment,
+                };
+
+                for planner in self.context_provider.get_expr_planners().iter() {
+                    match planner.plan_window(window_expr)? {
+                        PlannerResult::Planned(expr) => return Ok(expr),
+                        PlannerResult::Original(expr) => window_expr = expr,
+                    }
+                }
+
+                let RawWindowExpr {
+                    func_def,
+                    args,
+                    partition_by,
+                    order_by,
+                    window_frame,
+                    null_treatment,
+                } = window_expr;
+
+                return Expr::WindowFunction(expr::WindowFunction::new(func_def, args))
+                    .partition_by(partition_by)
+                    .order_by(order_by)
+                    .window_frame(window_frame)
+                    .null_treatment(null_treatment)
+                    .build();
             }
         } else {
             // User defined aggregate functions (UDAF) have precedence in case it has the same name as a scalar built-in function
@@ -341,8 +362,33 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
                     .map(|e| self.sql_expr_to_logical_expr(*e, schema, planner_context))
                     .transpose()?
                     .map(Box::new);
+
+                let mut aggregate_expr = RawAggregateExpr {
+                    func: fm,
+                    args,
+                    distinct,
+                    filter,
+                    order_by,
+                    null_treatment,
+                };
+                for planner in self.context_provider.get_expr_planners().iter() {
+                    match planner.plan_aggregate(aggregate_expr)? {
+                        PlannerResult::Planned(expr) => return Ok(expr),
+                        PlannerResult::Original(expr) => aggregate_expr = expr,
+                    }
+                }
+
+                let RawAggregateExpr {
+                    func,
+                    args,
+                    distinct,
+                    filter,
+                    order_by,
+                    null_treatment,
+                } = aggregate_expr;
+
                 return Ok(Expr::AggregateFunction(expr::AggregateFunction::new_udf(
-                    fm,
+                    func,
                     args,
                     distinct,
                     filter,
@@ -351,12 +397,21 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
                 )));
             }
         }
-
         // Could not find the relevant function, so return an error
         if let Some(suggested_func_name) =
             suggest_valid_function(&name, is_function_window, self.context_provider)
         {
             plan_err!("Invalid function '{name}'.\nDid you mean '{suggested_func_name}'?")
+                .map_err(|e| {
+                    let span = Span::try_from_sqlparser_span(sql_parser_span);
+                    let mut diagnostic =
+                        Diagnostic::new_error(format!("Invalid function '{name}'"), span);
+                    diagnostic.add_note(
+                        format!("Possible function '{}'", suggested_func_name),
+                        None,
+                    );
+                    e.with_diagnostic(diagnostic)
+                })
         } else {
             internal_err!("No functions registered with this context.")
         }
