@@ -18,7 +18,9 @@
 use crate::planner::{ContextProvider, PlannerContext, SqlToRel};
 use datafusion_common::{not_impl_err, Column, Result};
 use datafusion_expr::{JoinType, LogicalPlan, LogicalPlanBuilder};
-use sqlparser::ast::{Join, JoinConstraint, JoinOperator, TableFactor, TableWithJoins};
+use sqlparser::ast::{
+    Join, JoinConstraint, JoinOperator, ObjectName, TableFactor, TableWithJoins,
+};
 use std::collections::HashSet;
 
 impl<S: ContextProvider> SqlToRel<'_, S> {
@@ -123,11 +125,22 @@ impl<S: ContextProvider> SqlToRel<'_, S> {
                     .join_on(right, join_type, Some(expr))?
                     .build()
             }
-            JoinConstraint::Using(idents) => {
-                let keys: Vec<Column> = idents
+            JoinConstraint::Using(object_names) => {
+                let keys = object_names
                     .into_iter()
-                    .map(|x| Column::from_name(self.ident_normalizer.normalize(x)))
-                    .collect();
+                    .map(|object_name| {
+                        let ObjectName(mut object_names) = object_name;
+                        if object_names.len() != 1 {
+                            not_impl_err!(
+                                "Invalid identifier in USING clause. Expected single identifier, got {}", ObjectName(object_names)
+                            )
+                        } else {
+                            let id = object_names.swap_remove(0);
+                            Ok(self.ident_normalizer.normalize(id))
+                        }
+                    })
+                    .collect::<Result<Vec<_>>>()?;
+
                 LogicalPlanBuilder::from(left)
                     .join_using(right, join_type, keys)?
                     .build()
