@@ -15,11 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::collections::HashMap;
+
+use crate::variation_const::{
+    DEFAULT_CONTAINER_TYPE_VARIATION_REF, LARGE_CONTAINER_TYPE_VARIATION_REF,
+    VIEW_CONTAINER_TYPE_VARIATION_REF,
+};
+
 use datafusion::arrow::datatypes::DataType;
-use datafusion::datasource::physical_plan::ParquetExec;
+use datafusion::datasource::source::DataSourceExec;
 use datafusion::error::{DataFusionError, Result};
 use datafusion::physical_plan::{displayable, ExecutionPlan};
-use std::collections::HashMap;
+
+use datafusion::datasource::physical_plan::{FileScanConfig, ParquetSource};
 use substrait::proto::expression::mask_expression::{StructItem, StructSelect};
 use substrait::proto::expression::MaskExpression;
 use substrait::proto::r#type::{
@@ -35,11 +43,6 @@ use substrait::proto::ReadRel;
 use substrait::proto::Rel;
 use substrait::proto::{extensions, NamedStruct, Type};
 
-use crate::variation_const::{
-    DEFAULT_CONTAINER_TYPE_VARIATION_REF, LARGE_CONTAINER_TYPE_VARIATION_REF,
-    VIEW_CONTAINER_TYPE_VARIATION_REF,
-};
-
 /// Convert DataFusion ExecutionPlan to Substrait Rel
 pub fn to_substrait_rel(
     plan: &dyn ExecutionPlan,
@@ -48,87 +51,96 @@ pub fn to_substrait_rel(
         HashMap<String, u32>,
     ),
 ) -> Result<Box<Rel>> {
-    if let Some(scan) = plan.as_any().downcast_ref::<ParquetExec>() {
-        let base_config = scan.base_config();
-        let mut substrait_files = vec![];
-        for (partition_index, files) in base_config.file_groups.iter().enumerate() {
-            for file in files {
-                substrait_files.push(FileOrFiles {
-                    partition_index: partition_index.try_into().unwrap(),
-                    start: 0,
-                    length: file.object_meta.size as u64,
-                    path_type: Some(PathType::UriPath(
-                        file.object_meta.location.as_ref().to_string(),
-                    )),
-                    file_format: Some(FileFormat::Parquet(ParquetReadOptions {})),
-                });
-            }
-        }
-
-        let mut names = vec![];
-        let mut types = vec![];
-
-        for field in base_config.file_schema.fields.iter() {
-            match to_substrait_type(field.data_type(), field.is_nullable()) {
-                Ok(t) => {
-                    names.push(field.name().clone());
-                    types.push(t);
+    if let Some(data_source_exec) = plan.as_any().downcast_ref::<DataSourceExec>() {
+        let data_source = data_source_exec.data_source();
+        if let Some(file_config) = data_source.as_any().downcast_ref::<FileScanConfig>() {
+            let is_parquet = file_config
+                .file_source()
+                .as_any()
+                .downcast_ref::<ParquetSource>()
+                .is_some();
+            if is_parquet {
+                let mut substrait_files = vec![];
+                for (partition_index, files) in file_config.file_groups.iter().enumerate()
+                {
+                    for file in files {
+                        substrait_files.push(FileOrFiles {
+                            partition_index: partition_index.try_into().unwrap(),
+                            start: 0,
+                            length: file.object_meta.size as u64,
+                            path_type: Some(PathType::UriPath(
+                                file.object_meta.location.as_ref().to_string(),
+                            )),
+                            file_format: Some(FileFormat::Parquet(ParquetReadOptions {})),
+                        });
+                    }
                 }
-                Err(e) => return Err(e),
+
+                let mut names = vec![];
+                let mut types = vec![];
+
+                for field in file_config.file_schema.fields.iter() {
+                    match to_substrait_type(field.data_type(), field.is_nullable()) {
+                        Ok(t) => {
+                            names.push(field.name().clone());
+                            types.push(t);
+                        }
+                        Err(e) => return Err(e),
+                    }
+                }
+
+                let type_info = Struct {
+                    types,
+                    // FIXME: duckdb doesn't set this field, keep it as default variant 0.
+                    // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1106-L1127
+                    type_variation_reference: 0,
+                    nullability: Nullability::Required.into(),
+                };
+
+                let mut select_struct = None;
+                if let Some(projection) = file_config.projection.as_ref() {
+                    let struct_items = projection
+                        .iter()
+                        .map(|index| StructItem {
+                            field: *index as i32,
+                            // FIXME: duckdb sets this to None, but it's not clear why.
+                            // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1191
+                            child: None,
+                        })
+                        .collect();
+
+                    select_struct = Some(StructSelect { struct_items });
+                }
+
+                return Ok(Box::new(Rel {
+                    rel_type: Some(RelType::Read(Box::new(ReadRel {
+                        common: None,
+                        base_schema: Some(NamedStruct {
+                            names,
+                            r#struct: Some(type_info),
+                        }),
+                        filter: None,
+                        best_effort_filter: None,
+                        projection: Some(MaskExpression {
+                            select: select_struct,
+                            // FIXME: duckdb set this to true, but it's not clear why.
+                            // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1186.
+                            maintain_singular_struct: true,
+                        }),
+                        advanced_extension: None,
+                        read_type: Some(ReadType::LocalFiles(LocalFiles {
+                            items: substrait_files,
+                            advanced_extension: None,
+                        })),
+                    }))),
+                }));
             }
         }
-
-        let type_info = Struct {
-            types,
-            // FIXME: duckdb doesn't set this field, keep it as default variant 0.
-            // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1106-L1127
-            type_variation_reference: 0,
-            nullability: Nullability::Required.into(),
-        };
-
-        let mut select_struct = None;
-        if let Some(projection) = base_config.projection.as_ref() {
-            let struct_items = projection
-                .iter()
-                .map(|index| StructItem {
-                    field: *index as i32,
-                    // FIXME: duckdb sets this to None, but it's not clear why.
-                    // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1191
-                    child: None,
-                })
-                .collect();
-
-            select_struct = Some(StructSelect { struct_items });
-        }
-
-        Ok(Box::new(Rel {
-            rel_type: Some(RelType::Read(Box::new(ReadRel {
-                common: None,
-                base_schema: Some(NamedStruct {
-                    names,
-                    r#struct: Some(type_info),
-                }),
-                filter: None,
-                best_effort_filter: None,
-                projection: Some(MaskExpression {
-                    select: select_struct,
-                    // FIXME: duckdb set this to true, but it's not clear why.
-                    // https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L1186.
-                    maintain_singular_struct: true,
-                }),
-                advanced_extension: None,
-                read_type: Some(ReadType::LocalFiles(LocalFiles {
-                    items: substrait_files,
-                    advanced_extension: None,
-                })),
-            }))),
-        }))
-    } else {
-        Err(DataFusionError::Substrait(format!(
-            "Unsupported plan in Substrait physical plan producer: {}",
-            displayable(plan).one_line()
-        )))
     }
+    Err(DataFusionError::Substrait(format!(
+        "Unsupported plan in Substrait physical plan producer: {}",
+        displayable(plan).one_line()
+    )))
 }
 
 // see https://github.com/duckdb/substrait/blob/b6f56643cb11d52de0e32c24a01dfd5947df62be/src/to_substrait.cpp#L954-L1094.
