@@ -32,8 +32,8 @@ use datafusion::execution::registry::SerializerRegistry;
 use datafusion::execution::runtime_env::RuntimeEnv;
 use datafusion::execution::session_state::SessionStateBuilder;
 use datafusion::logical_expr::{
-    Extension, LogicalPlan, PartitionEvaluator, Repartition, UserDefinedLogicalNode,
-    Values, Volatility,
+    Extension, InvariantLevel, LogicalPlan, PartitionEvaluator, Repartition,
+    UserDefinedLogicalNode, Values, Volatility,
 };
 use datafusion::optimizer::simplify_expressions::expr_simplifier::THRESHOLD_INLINE_INLIST;
 use datafusion::prelude::*;
@@ -111,6 +111,14 @@ impl UserDefinedLogicalNode for MockUserDefinedLogicalPlan {
         &self.empty_schema
     }
 
+    fn check_invariants(
+        &self,
+        _check: InvariantLevel,
+        _plan: &LogicalPlan,
+    ) -> Result<()> {
+        Ok(())
+    }
+
     fn expressions(&self) -> Vec<Expr> {
         vec![]
     }
@@ -300,6 +308,17 @@ async fn aggregate_grouping_rollup() -> Result<()> {
     ).await
 }
 
+#[tokio::test]
+async fn multilayer_aggregate() -> Result<()> {
+    assert_expected_plan(
+        "SELECT a, sum(partial_count_b) FROM (SELECT a, count(b) as partial_count_b FROM data GROUP BY a) GROUP BY a",
+        "Aggregate: groupBy=[[data.a]], aggr=[[sum(count(data.b)) AS sum(partial_count_b)]]\
+        \n  Aggregate: groupBy=[[data.a]], aggr=[[count(data.b)]]\
+        \n    TableScan: data projection=[a, b]",
+        true
+    ).await
+}
+
 #[tokio::test]
 async fn decimal_literal() -> Result<()> {
     roundtrip("SELECT * FROM data WHERE b > 2.5").await
@@ -571,6 +590,21 @@ async fn roundtrip_self_implicit_cross_join() -> Result<()> {
     roundtrip("SELECT left.a left_a, left.b, right.a right_a, right.c FROM data AS left, data AS right").await
 }
 
+#[tokio::test]
+async fn self_join_introduces_aliases() -> Result<()> {
+    assert_expected_plan(
+        "SELECT d1.b, d2.c FROM data d1 JOIN data d2 ON d1.b = d2.b",
+        "Projection: left.b, right.c\
+        \n  Inner Join: left.b = right.b\
+        \n    SubqueryAlias: left\
+        \n      TableScan: data projection=[b]\
+        \n    SubqueryAlias: right\
+        \n      TableScan: data projection=[b, c]",
+        false,
+    )
+    .await
+}
+
 #[tokio::test]
 async fn roundtrip_arithmetic_ops() -> Result<()> {
     roundtrip("SELECT a - a FROM data").await?;
@@ -661,18 +695,51 @@ async fn roundtrip_union_all() -> Result<()> {
 
 #[tokio::test]
 async fn simple_intersect() -> Result<()> {
-    // Substrait treats both count(*) and count(1) the same
-    assert_expected_plan(
-        "SELECT count(*) FROM (SELECT data.a FROM data INTERSECT SELECT data2.a FROM data2);",
-        "Aggregate: groupBy=[[]], aggr=[[count(Int64(1)) AS count(*)]]\
-         \n  Projection: \
-         \n    LeftSemi Join: data.a = data2.a\
-         \n      Aggregate: groupBy=[[data.a]], aggr=[[]]\
-         \n        TableScan: data projection=[a]\
-         \n      TableScan: data2 projection=[a]",
-        true
+    async fn check_wildcard(syntax: &str) -> Result<()> {
+        let expected_plan_str = format!(
+            "Projection: count(Int64(1)) AS {syntax}\
+        \n  Aggregate: groupBy=[[]], aggr=[[count(Int64(1))]]\
+        \n    Projection: \
+        \n      LeftSemi Join: data.a = data2.a\
+        \n        Aggregate: groupBy=[[data.a]], aggr=[[]]\
+        \n          TableScan: data projection=[a]\
+        \n        TableScan: data2 projection=[a]"
+        );
+
+        assert_expected_plan(
+            &format!("SELECT {syntax} FROM (SELECT data.a FROM data INTERSECT SELECT data2.a FROM data2);"),
+            &expected_plan_str,
+            true
+        ).await
+    }
+
+    async fn check_constant(sql_syntax: &str, plan_expr: &str) -> Result<()> {
+        let expected_plan_str = format!(
+            "Aggregate: groupBy=[[]], aggr=[[{plan_expr}]]\
+        \n  Projection: \
+        \n    LeftSemi Join: data.a = data2.a\
+        \n      Aggregate: groupBy=[[data.a]], aggr=[[]]\
+        \n        TableScan: data projection=[a]\
+        \n      TableScan: data2 projection=[a]"
+        );
+
+        assert_expected_plan(
+            &format!("SELECT {sql_syntax} FROM (SELECT data.a FROM data INTERSECT SELECT data2.a FROM data2);"),
+            &expected_plan_str,
+            true
+        ).await
+    }
+
+    check_wildcard("count(*)").await?;
+    check_wildcard("count()").await?;
+    check_constant("count(1)", "count(Int64(1))").await?;
+    check_constant("count(2)", "count(Int64(2))").await?;
+    check_constant(
+        "count(1 + 2)",
+        "count(Int64(3)) AS count(Int64(1) + Int64(2))",
     )
-        .await
+    .await?;
+    Ok(())
 }
 
 #[tokio::test]
@@ -797,18 +864,57 @@ async fn simple_intersect_table_reuse() -> Result<()> {
     // Instead, when we consume Substrait, we add aliases before a join that'd otherwise collide.
     // In this case the aliasing happens at a different point in the plan, so we cannot use roundtrip.
     // Schema check works because we set aliases to what the Substrait consumer will generate.
-    assert_expected_plan(
-        "SELECT count(1) FROM (SELECT left.a FROM data AS left INTERSECT SELECT right.a FROM data AS right);",
-        "Aggregate: groupBy=[[]], aggr=[[count(Int64(1))]]\
+
+    async fn check_wildcard(syntax: &str) -> Result<()> {
+        let expected_plan_str = format!(
+            "Projection: count(Int64(1)) AS {syntax}\
+        \n  Aggregate: groupBy=[[]], aggr=[[count(Int64(1))]]\
+        \n    Projection: \
+        \n      LeftSemi Join: left.a = right.a\
+        \n        SubqueryAlias: left\
+        \n          Aggregate: groupBy=[[data.a]], aggr=[[]]\
+        \n            TableScan: data projection=[a]\
+        \n        SubqueryAlias: right\
+        \n          TableScan: data projection=[a]"
+        );
+
+        assert_expected_plan(
+            &format!("SELECT {syntax} FROM (SELECT left.a FROM data AS left INTERSECT SELECT right.a FROM data AS right);"),
+            &expected_plan_str,
+            true
+        ).await
+    }
+
+    async fn check_constant(sql_syntax: &str, plan_expr: &str) -> Result<()> {
+        let expected_plan_str = format!(
+            "Aggregate: groupBy=[[]], aggr=[[{plan_expr}]]\
         \n  Projection: \
         \n    LeftSemi Join: left.a = right.a\
         \n      SubqueryAlias: left\
         \n        Aggregate: groupBy=[[data.a]], aggr=[[]]\
         \n          TableScan: data projection=[a]\
         \n      SubqueryAlias: right\
-        \n        TableScan: data projection=[a]",
-        true
-    ).await
+        \n        TableScan: data projection=[a]"
+        );
+
+        assert_expected_plan(
+            &format!("SELECT {sql_syntax} FROM (SELECT left.a FROM data AS left INTERSECT SELECT right.a FROM data AS right);"),
+            &expected_plan_str,
+            true
+        ).await
+    }
+
+    check_wildcard("count(*)").await?;
+    check_wildcard("count()").await?;
+    check_constant("count(1)", "count(Int64(1))").await?;
+    check_constant("count(2)", "count(Int64(2))").await?;
+    check_constant(
+        "count(1 + 2)",
+        "count(Int64(3)) AS count(Int64(1) + Int64(2))",
+    )
+    .await?;
+
+    Ok(())
 }
 
 #[tokio::test]
